@@ -1,5 +1,4 @@
 //go:build !linux
-// +build !linux
 
 package cgmgr
 
@@ -10,6 +9,7 @@ import (
 type CgroupStats struct {
 	Memory     *MemoryStats
 	CPU        *CPUStats
+	Hugetlb    map[string]HugetlbStats
 	Pid        *PidsStats
 	SystemNano int64
 }
@@ -45,6 +45,11 @@ type CPUStats struct {
 	ThrottledTime uint64
 }
 
+type HugetlbStats struct {
+	Usage uint64
+	Max   uint64
+}
+
 type PidsStats struct {
 	Current uint64
 	Limit   uint64
