@@ -1,8 +1,11 @@
 //go:build !linux
-// +build !linux
 
 package node
 
+func CgroupIsV2() bool {
+	return false
+}
+
 // CgroupHasMemorySwap returns whether the memory swap controller is present
 func CgroupHasMemorySwap() bool {
 	return false
