@@ -7,22 +7,24 @@ import (
 	"strconv"
 	"time"
 
+	"github.com/moby/sys/capability"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	v1 "github.com/opencontainers/image-spec/specs-go/v1"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
-	validate "github.com/opencontainers/runtime-tools/validate/capabilities"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 	kubeletTypes "k8s.io/kubelet/pkg/types"
 
 	"github.com/cri-o/cri-o/internal/config/capabilities"
 	"github.com/cri-o/cri-o/internal/hostport"
-	"github.com/cri-o/cri-o/internal/lib"
+	"github.com/cri-o/cri-o/internal/lib/constants"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
+	"github.com/cri-o/cri-o/internal/memorystore"
 	oci "github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/storage"
 	"github.com/cri-o/cri-o/internal/storage/references"
 	"github.com/cri-o/cri-o/pkg/annotations"
+	pkgConfig "github.com/cri-o/cri-o/pkg/config"
 )
 
 var _ = t.Describe("Container", func() {
@@ -89,19 +91,44 @@ var _ = t.Describe("Container", func() {
 			volumes := []oci.ContainerVolume{}
 			imageID, err := storage.ParseStorageImageIDFromOutOfProcessData("8a788232037eaf17794408ff3df6b922a1aedf9ef8de36afdae3ed0b0381907b")
 			Expect(err).ToNot(HaveOccurred())
-			imageName, err := references.ParseRegistryImageReferenceFromOutOfProcessData("example.com/repo/image:tag")
+			someNameOfThisImage, err := references.ParseRegistryImageReferenceFromOutOfProcessData("example.com/repo/image:tag")
 			Expect(err).ToNot(HaveOccurred())
 			imageResult := storage.ImageResult{
 				ID:                  imageID,
-				SomeNameOfThisImage: &imageName,
+				SomeNameOfThisImage: &someNameOfThisImage,
 			}
 			mountPoint := "test"
 			configStopSignal := "test"
 
-			sb, err := sandbox.New("sandboxID", "", "", "", "test",
-				make(map[string]string), make(map[string]string), "", "",
-				&types.PodSandboxMetadata{}, "", "", false, "", "", "",
-				[]*hostport.PortMapping{}, false, currentTime, "", nil, nil)
+			sbox := sandbox.NewBuilder()
+
+			sbox.SetID("sandboxID")
+			sbox.SetName("sandboxName")
+			sbox.SetLogDir("test")
+			sbox.SetShmPath("test")
+			sbox.SetNamespace("")
+			sbox.SetKubeName("")
+			sbox.SetMountLabel("test")
+			sbox.SetProcessLabel("test")
+			sbox.SetCgroupParent("")
+			sbox.SetPrivileged(false)
+			sbox.SetRuntimeHandler("")
+			sbox.SetResolvPath("")
+			sbox.SetHostname("")
+			sbox.SetPortMappings([]*hostport.PortMapping{})
+			sbox.SetHostNetwork(false)
+			sbox.SetUsernsMode("")
+			sbox.SetPodLinuxOverhead(nil)
+			sbox.SetPodLinuxResources(nil)
+			sbox.SetCreatedAt(time.Now())
+			err = sbox.SetCRISandbox(sbox.ID(), make(map[string]string), make(map[string]string), &types.PodSandboxMetadata{})
+			Expect(err).ToNot(HaveOccurred())
+			sbox.SetPortMappings([]*hostport.PortMapping{})
+			sbox.SetHostNetwork(false)
+			sbox.SetContainers(memorystore.New[*oci.Container]())
+			sbox.SetCreatedAt(currentTime)
+
+			sb, err := sbox.GetSandbox()
 			Expect(err).ToNot(HaveOccurred())
 
 			image, err := sut.UserRequestedImage()
@@ -110,16 +137,16 @@ var _ = t.Describe("Container", func() {
 			logpath, err := sut.LogPath(sb.LogDir())
 			Expect(err).ToNot(HaveOccurred())
 
-			metadataJSON, err := json.Marshal(sut.Config().Metadata)
+			metadataJSON, err := json.Marshal(sut.Config().GetMetadata())
 			Expect(err).ToNot(HaveOccurred())
 
-			labelsJSON, err := json.Marshal(sut.Config().Labels)
+			labelsJSON, err := json.Marshal(sut.Config().GetLabels())
 			Expect(err).ToNot(HaveOccurred())
 
 			volumesJSON, err := json.Marshal(volumes)
 			Expect(err).ToNot(HaveOccurred())
 
-			kubeAnnotationsJSON, err := json.Marshal(sut.Config().Annotations)
+			kubeAnnotationsJSON, err := json.Marshal(sut.Config().GetAnnotations())
 			Expect(err).ToNot(HaveOccurred())
 
 			Expect(currentTime).ToNot(BeNil())
@@ -128,8 +155,8 @@ var _ = t.Describe("Container", func() {
 			err = sut.SpecAddAnnotations(context.Background(), sb, volumes, mountPoint, configStopSignal, &imageResult, false, "foo", "")
 			Expect(err).ToNot(HaveOccurred())
 
-			Expect(sut.Spec().Config.Annotations[annotations.Image]).To(Equal(image))
-			Expect(sut.Spec().Config.Annotations[annotations.ImageName]).To(Equal(imageResult.SomeNameOfThisImage.StringForOutOfProcessConsumptionOnly()))
+			Expect(sut.Spec().Config.Annotations[annotations.UserRequestedImage]).To(Equal(image))
+			Expect(sut.Spec().Config.Annotations[annotations.SomeNameOfTheImage]).To(Equal(imageResult.SomeNameOfThisImage.StringForOutOfProcessConsumptionOnly()))
 			Expect(sut.Spec().Config.Annotations[annotations.ImageRef]).To(Equal(imageResult.ID.IDStringForOutOfProcessConsumptionOnly()))
 			Expect(sut.Spec().Config.Annotations[annotations.Name]).To(Equal(sut.Name()))
 			Expect(sut.Spec().Config.Annotations[annotations.ContainerID]).To(Equal(sut.ID()))
@@ -137,11 +164,11 @@ var _ = t.Describe("Container", func() {
 			Expect(sut.Spec().Config.Annotations[annotations.SandboxName]).To(Equal(sb.Name()))
 			Expect(sut.Spec().Config.Annotations[annotations.ContainerType]).To(Equal(annotations.ContainerTypeContainer))
 			Expect(sut.Spec().Config.Annotations[annotations.LogPath]).To(Equal(logpath))
-			Expect(sut.Spec().Config.Annotations[annotations.TTY]).To(Equal(strconv.FormatBool(sut.Config().Tty)))
-			Expect(sut.Spec().Config.Annotations[annotations.Stdin]).To(Equal(strconv.FormatBool(sut.Config().Stdin)))
-			Expect(sut.Spec().Config.Annotations[annotations.StdinOnce]).To(Equal(strconv.FormatBool(sut.Config().StdinOnce)))
+			Expect(sut.Spec().Config.Annotations[annotations.TTY]).To(Equal(strconv.FormatBool(sut.Config().GetTty())))
+			Expect(sut.Spec().Config.Annotations[annotations.Stdin]).To(Equal(strconv.FormatBool(sut.Config().GetStdin())))
+			Expect(sut.Spec().Config.Annotations[annotations.StdinOnce]).To(Equal(strconv.FormatBool(sut.Config().GetStdinOnce())))
 			Expect(sut.Spec().Config.Annotations[annotations.ResolvPath]).To(Equal(sb.ResolvPath()))
-			Expect(sut.Spec().Config.Annotations[annotations.ContainerManager]).To(Equal(lib.ContainerManagerCRIO))
+			Expect(sut.Spec().Config.Annotations[annotations.ContainerManager]).To(Equal(constants.ContainerManagerCRIO))
 			Expect(sut.Spec().Config.Annotations[annotations.MountPoint]).To(Equal(mountPoint))
 			Expect(sut.Spec().Config.Annotations[annotations.SeccompProfilePath]).To(Equal("foo"))
 			Expect(sut.Spec().Config.Annotations[annotations.Created]).ToNot(BeNil())
@@ -397,7 +424,7 @@ var _ = t.Describe("Container", func() {
 
 			// Then
 			Expect(sut.SpecSetProcessArgs(nil)).To(Succeed())
-			Expect(sut.Spec().Config.Process.Args).To(Equal(config.Command))
+			Expect(sut.Spec().Config.Process.Args).To(Equal(config.GetCommand()))
 		})
 		It("should set to Args", func() {
 			// Given
@@ -409,7 +436,7 @@ var _ = t.Describe("Container", func() {
 
 			// Then
 			Expect(sut.SpecSetProcessArgs(nil)).To(Succeed())
-			Expect(sut.Spec().Config.Process.Args).To(Equal(config.Args))
+			Expect(sut.Spec().Config.Process.Args).To(Equal(config.GetArgs()))
 		})
 		It("should append args and command", func() {
 			// Given
@@ -421,7 +448,7 @@ var _ = t.Describe("Container", func() {
 
 			// Then
 			Expect(sut.SpecSetProcessArgs(nil)).To(Succeed())
-			Expect(sut.Spec().Config.Process.Args).To(Equal(append(config.Command, config.Args...)))
+			Expect(sut.Spec().Config.Process.Args).To(Equal(append(config.Command, config.GetArgs()...)))
 		})
 		It("should inherit entrypoint from image", func() {
 			// Given
@@ -438,7 +465,7 @@ var _ = t.Describe("Container", func() {
 
 			// Then
 			Expect(sut.SpecSetProcessArgs(img)).To(Succeed())
-			Expect(sut.Spec().Config.Process.Args).To(Equal(append(img.Config.Entrypoint, config.Args...)))
+			Expect(sut.Spec().Config.Process.Args).To(Equal(append(img.Config.Entrypoint, config.GetArgs()...)))
 		})
 		It("should always use Command if specified", func() {
 			// Given
@@ -455,7 +482,7 @@ var _ = t.Describe("Container", func() {
 
 			// Then
 			Expect(sut.SpecSetProcessArgs(img)).To(Succeed())
-			Expect(sut.Spec().Config.Process.Args).To(Equal(config.Command))
+			Expect(sut.Spec().Config.Process.Args).To(Equal(config.GetCommand()))
 		})
 		It("should inherit cmd from image", func() {
 			// Given
@@ -592,7 +619,9 @@ var _ = t.Describe("Container", func() {
 			Expect(sut.SpecSetupCapabilities(caps, serverCaps, false)).To(Succeed())
 			// `int(validate.LastCap())+1` represents the total number of `ALL` capabilities
 			// in the current environment, while `-1` indicates the removal of `CHOWN` from `ALL`.
-			verifyCapValues(sut.Spec().Config.Process.Capabilities, int(validate.LastCap())+1-1)
+			lastCap, err := capability.LastCap()
+			Expect(err).NotTo(HaveOccurred())
+			verifyCapValues(sut.Spec().Config.Process.Capabilities, int(lastCap)+1-1)
 		})
 		It("AddCapabilities one DropCapabilities ALL should add that one", func() {
 			caps := &types.Capability{
@@ -633,4 +662,213 @@ var _ = t.Describe("Container", func() {
 			Expect(sut.Spec().Config.Process.Capabilities.Inheritable).To(HaveLen(1))
 		})
 	})
+	t.Describe("SpecSetPrivileges", func() {
+		It("Non privileged container should get selected capabilities", func() {
+			// Given
+			sc := &types.LinuxContainerSecurityContext{
+				Capabilities: &types.Capability{
+					AddCapabilities:  []string{"CHOWN"},
+					DropCapabilities: nil,
+				},
+			}
+			cfg := &pkgConfig.Config{}
+
+			// When
+			Expect(sut.SpecSetPrivileges(context.Background(), sc, cfg)).To(Succeed())
+
+			// Then
+			Expect(sut.Spec().Config.Process.Capabilities.Bounding).To(HaveLen(len(cfg.DefaultCapabilities) + 1))
+			Expect(sut.Spec().Config.Process.Capabilities.Effective).To(HaveLen(len(cfg.DefaultCapabilities) + 1))
+			Expect(sut.Spec().Config.Process.Capabilities.Permitted).To(HaveLen(len(cfg.DefaultCapabilities) + 1))
+			Expect(sut.Spec().Config.Process.Capabilities.Inheritable).To(BeEmpty())
+			Expect(sut.Spec().Config.Process.Capabilities.Ambient).To(BeEmpty())
+		})
+		It("Privileged container gets all capabilities", func() {
+			// Given
+			sc := &types.LinuxContainerSecurityContext{}
+			cfg := &pkgConfig.Config{}
+			config := &types.ContainerConfig{
+				Metadata: &types.ContainerMetadata{Name: "name"},
+				Linux: &types.LinuxContainerConfig{
+					SecurityContext: &types.LinuxContainerSecurityContext{
+						Privileged: true,
+					},
+				},
+			}
+			sboxConfig := &types.PodSandboxConfig{
+				Linux: &types.LinuxPodSandboxConfig{
+					SecurityContext: &types.LinuxSandboxSecurityContext{
+						Privileged: true,
+					},
+				},
+			}
+			expectedSize := len(capability.ListKnown())
+
+			// When
+			Expect(sut.SetConfig(config, sboxConfig)).To(Succeed())
+			Expect(sut.SetPrivileged()).To(Succeed())
+			Expect(sut.SpecSetPrivileges(context.Background(), sc, cfg)).To(Succeed())
+
+			// Then
+			Expect(sut.Spec().Config.Process.Capabilities.Bounding).To(HaveLen(expectedSize))
+			Expect(sut.Spec().Config.Process.Capabilities.Effective).To(HaveLen(expectedSize))
+			Expect(sut.Spec().Config.Process.Capabilities.Permitted).To(HaveLen(expectedSize))
+			Expect(sut.Spec().Config.Process.Capabilities.Inheritable).To(HaveLen(expectedSize))
+			Expect(sut.Spec().Config.Process.Capabilities.Ambient).To(HaveLen(expectedSize))
+		})
+		It("Should set NoNewPrivs flag if set", func() {
+			// Given
+			sc := &types.LinuxContainerSecurityContext{
+				NoNewPrivs: true,
+			}
+			cfg := &pkgConfig.Config{}
+
+			// When
+			Expect(sut.SpecSetPrivileges(context.Background(), sc, cfg)).To(Succeed())
+
+			// Then
+			Expect(sut.Spec().Config.Process.NoNewPrivileges).To(BeTrue())
+		})
+		It("Should add masked paths if set", func() {
+			// Given
+			sc := &types.LinuxContainerSecurityContext{
+				MaskedPaths: []string{"path1", "path2"},
+			}
+			cfg := &pkgConfig.Config{}
+
+			// When
+			Expect(sut.SpecSetPrivileges(context.Background(), sc, cfg)).To(Succeed())
+
+			// Then
+			Expect(sut.Spec().Config.Linux.MaskedPaths).To(HaveLen(2))
+		})
+		It("Should add readonly paths if set", func() {
+			// Given
+			sc := &types.LinuxContainerSecurityContext{
+				ReadonlyPaths: []string{"path1", "path2"},
+			}
+			cfg := &pkgConfig.Config{}
+
+			// When
+			Expect(sut.SpecSetPrivileges(context.Background(), sc, cfg)).To(Succeed())
+
+			// Then
+			Expect(sut.Spec().Config.Linux.ReadonlyPaths).To(HaveLen(2))
+		})
+	})
+	t.Describe("SpecSetLinuxContainerResources", func() {
+		It("Sets all fields to their expected values", func() {
+			// Given
+			resources := &types.LinuxContainerResources{
+				CpuPeriod:   1,
+				CpuQuota:    2,
+				CpuShares:   3,
+				OomScoreAdj: 4,
+				CpusetCpus:  "5",
+				CpusetMems:  "6",
+			}
+
+			// When
+			Expect(sut.SpecSetLinuxContainerResources(resources, 0)).To(Succeed())
+
+			// Then
+			Expect(*sut.Spec().Config.Linux.Resources.CPU.Period).To(Equal(uint64(resources.GetCpuPeriod())))
+			Expect(*sut.Spec().Config.Linux.Resources.CPU.Quota).To(Equal(resources.GetCpuQuota()))
+			Expect(*sut.Spec().Config.Linux.Resources.CPU.Shares).To(Equal(uint64(resources.GetCpuShares())))
+			Expect(*sut.Spec().Config.Process.OOMScoreAdj).To(Equal(int(resources.GetOomScoreAdj())))
+			Expect(sut.Spec().Config.Linux.Resources.CPU.Cpus).To(Equal(resources.GetCpusetCpus()))
+			Expect(sut.Spec().Config.Linux.Resources.CPU.Mems).To(Equal(resources.GetCpusetMems()))
+		})
+		It("Fails to set memory limit if invalid", func() {
+			// Given
+			minMemory := int64(2048)
+
+			// When
+			resources := &types.LinuxContainerResources{
+				MemoryLimitInBytes: 1024, // must be >= minMemory
+			}
+
+			// Then
+			Expect(sut.SpecSetLinuxContainerResources(resources, minMemory)).NotTo(Succeed())
+		})
+		It("Fails to set memory swap limit if invalid", func() {
+			// Given
+			minMemory := int64(2048)
+
+			// When
+			resources := &types.LinuxContainerResources{
+				MemoryLimitInBytes:     2048,
+				MemorySwapLimitInBytes: 1024, // must be >= MemoryLimitInBytes
+			}
+
+			// Then
+			Expect(sut.SpecSetLinuxContainerResources(resources, minMemory)).NotTo(Succeed())
+		})
+		It("Set memory limit to both swap and RAM when only MemoryLimit is set", func() {
+			// Given
+			resources := &types.LinuxContainerResources{
+				MemoryLimitInBytes: 4096,
+			}
+
+			// When
+			Expect(sut.SpecSetLinuxContainerResources(resources, 2048)).To(Succeed())
+
+			// Then
+			Expect(*sut.Spec().Config.Linux.Resources.Memory.Limit).To(Equal(resources.GetMemoryLimitInBytes()))
+			Expect(*sut.Spec().Config.Linux.Resources.Memory.Swap).To(Equal(resources.GetMemoryLimitInBytes()))
+		})
+		It("Set memory limits appropriately when Limit and SwapLimit are set", func() {
+			// Given
+			resources := &types.LinuxContainerResources{
+				MemoryLimitInBytes:     4096,
+				MemorySwapLimitInBytes: 4096,
+			}
+
+			// When
+			Expect(sut.SpecSetLinuxContainerResources(resources, 0)).To(Succeed())
+
+			// Then
+			Expect(*sut.Spec().Config.Linux.Resources.Memory.Limit).To(Equal(resources.GetMemoryLimitInBytes()))
+			Expect(*sut.Spec().Config.Linux.Resources.Memory.Swap).To(Equal(resources.GetMemorySwapLimitInBytes()))
+		})
+		It("Set hugepage limits", func() {
+			// Given
+			hugepageLimits := []*types.HugepageLimit{
+				{
+					PageSize: "1KB",
+					Limit:    1024,
+				},
+				{
+					PageSize: "2KB",
+					Limit:    2048,
+				},
+			}
+			resources := &types.LinuxContainerResources{
+				HugepageLimits: hugepageLimits,
+			}
+
+			// When
+			Expect(sut.SpecSetLinuxContainerResources(resources, 0)).To(Succeed())
+
+			// Then
+			for i, pageLimit := range sut.Spec().Config.Linux.Resources.HugepageLimits {
+				Expect(pageLimit.Pagesize).To(Equal(hugepageLimits[i].GetPageSize()))
+				Expect(pageLimit.Limit).To(Equal(hugepageLimits[i].GetLimit()))
+			}
+		})
+		It("Set Cgroupv2 resources", func() {
+			// Given
+			resources := &types.LinuxContainerResources{
+				Unified: make(map[string]string, 2),
+			}
+			resources.Unified["memory.high"] = "8000000"
+			resources.Unified["memory.low"] = "100000"
+
+			// When
+			Expect(sut.SpecSetLinuxContainerResources(resources, 2048)).To(Succeed())
+
+			// Then
+			Expect(sut.Spec().Config.Linux.Resources.Unified).To(HaveLen(len(resources.GetUnified())))
+		})
+	})
 })
