@@ -33,9 +33,10 @@ import (
 	"k8s.io/apimachinery/pkg/util/sets"
 	utilversion "k8s.io/apimachinery/pkg/util/version"
 	utilwait "k8s.io/apimachinery/pkg/util/wait"
-	"k8s.io/klog/v2"
 	utilexec "k8s.io/utils/exec"
 	utiltrace "k8s.io/utils/trace"
+
+	"github.com/cri-o/cri-o/internal/log"
 )
 
 // RulePosition holds the -I/-A flags for iptable.
@@ -218,16 +219,11 @@ type runner struct {
 
 // newInternal returns a new Interface which will exec iptables, and allows the
 // caller to change the iptables-restore lockfile path.
-func newInternal(exec utilexec.Interface, protocol Protocol, lockfilePath14x, lockfilePath16x string) Interface {
-	version, err := getIPTablesVersion(exec, protocol)
-	if err != nil {
-		klog.InfoS("Error checking iptables version, assuming version at least", "version", MinCheckVersion, "err", err)
-		version = MinCheckVersion
-	}
-
+func newInternal(ctx context.Context, exec utilexec.Interface, protocol Protocol, lockfilePath14x, lockfilePath16x string) Interface {
 	if lockfilePath16x == "" {
 		lockfilePath16x = LockfilePath16x
 	}
+
 	if lockfilePath14x == "" {
 		lockfilePath14x = LockfilePath14x
 	}
@@ -235,19 +231,31 @@ func newInternal(exec utilexec.Interface, protocol Protocol, lockfilePath14x, lo
 	runner := &runner{
 		exec:            exec,
 		protocol:        protocol,
-		hasCheck:        version.AtLeast(MinCheckVersion),
-		hasRandomFully:  version.AtLeast(RandomFullyMinVersion),
-		waitFlag:        getIPTablesWaitFlag(version),
-		restoreWaitFlag: getIPTablesRestoreWaitFlag(version, exec, protocol),
 		lockfilePath14x: lockfilePath14x,
 		lockfilePath16x: lockfilePath16x,
 	}
+
+	version, err := getIPTablesVersion(exec, protocol)
+	if err != nil {
+		// The only likely error is "no such file or directory", in which case any
+		// further commands will fail the same way, so we don't need to do
+		// anything special here.
+		log.Debugf(ctx, "Error checking iptables version: %v", err)
+
+		return runner
+	}
+
+	runner.hasCheck = version.AtLeast(MinCheckVersion)
+	runner.hasRandomFully = version.AtLeast(RandomFullyMinVersion)
+	runner.waitFlag = getIPTablesWaitFlag(version)
+	runner.restoreWaitFlag = getIPTablesRestoreWaitFlag(version, exec, protocol)
+
 	return runner
 }
 
 // New returns a new Interface which will exec iptables.
-func New(exec utilexec.Interface, protocol Protocol) Interface {
-	return newInternal(exec, protocol, "", "")
+func New(ctx context.Context, exec utilexec.Interface, protocol Protocol) Interface {
+	return newInternal(ctx, exec, protocol, "", "")
 }
 
 // EnsureChain is part of Interface.
@@ -265,8 +273,10 @@ func (runner *runner) EnsureChain(table Table, chain Chain) (bool, error) {
 				return true, nil
 			}
 		}
+
 		return false, fmt.Errorf("error creating chain %q: %w: %s", chain, err, out)
 	}
+
 	return false, nil
 }
 
@@ -281,6 +291,7 @@ func (runner *runner) FlushChain(table Table, chain Chain) error {
 	if err != nil {
 		return fmt.Errorf("error flushing chain %q: %w: %s", chain, err, out)
 	}
+
 	return nil
 }
 
@@ -295,6 +306,7 @@ func (runner *runner) DeleteChain(table Table, chain Chain) error {
 	if err != nil {
 		return fmt.Errorf("error deleting chain %q: %w: %s", chain, err, out)
 	}
+
 	return nil
 }
 
@@ -309,13 +321,16 @@ func (runner *runner) EnsureRule(position RulePosition, table Table, chain Chain
 	if err != nil {
 		return false, err
 	}
+
 	if exists {
 		return true, nil
 	}
+
 	out, err := runner.run(operation(position), fullArgs)
 	if err != nil {
 		return false, fmt.Errorf("error appending rule: %w: %s", err, out)
 	}
+
 	return false, nil
 }
 
@@ -330,13 +345,16 @@ func (runner *runner) DeleteRule(table Table, chain Chain, args ...string) error
 	if err != nil {
 		return err
 	}
+
 	if !exists {
 		return nil
 	}
+
 	out, err := runner.run(opDeleteRule, fullArgs)
 	if err != nil {
 		return fmt.Errorf("error deleting rule: %w: %s", err, out)
 	}
+
 	return nil
 }
 
@@ -359,9 +377,10 @@ func (runner *runner) SaveInto(table Table, buffer *bytes.Buffer) error {
 	// run and return
 	iptablesSaveCmd := iptablesSaveCommand(runner.protocol)
 	args := []string{"-t", string(table)}
-	klog.V(4).InfoS("Running", "command", iptablesSaveCmd, "arguments", args)
+	log.Debugf(context.Background(), "Running (command=%q arguments=%+v)", iptablesSaveCmd, args)
 	cmd := runner.exec.Command(iptablesSaveCmd, args...)
 	cmd.SetStdout(buffer)
+
 	stderrBuffer := bytes.NewBuffer(nil)
 	cmd.SetStderr(stderrBuffer)
 
@@ -370,6 +389,7 @@ func (runner *runner) SaveInto(table Table, buffer *bytes.Buffer) error {
 		//nolint:errcheck
 		stderrBuffer.WriteTo(buffer) // ignore error, since we need to return the original error
 	}
+
 	return err
 }
 
@@ -377,6 +397,7 @@ func (runner *runner) SaveInto(table Table, buffer *bytes.Buffer) error {
 func (runner *runner) Restore(table Table, data []byte, flush FlushFlag, counters RestoreCountersFlag) error {
 	// setup args
 	args := []string{"-T", string(table)}
+
 	return runner.restoreInternal(args, data, flush, counters)
 }
 
@@ -384,6 +405,7 @@ func (runner *runner) Restore(table Table, data []byte, flush FlushFlag, counter
 func (runner *runner) RestoreAll(data []byte, flush FlushFlag, counters RestoreCountersFlag) error {
 	// setup args
 	args := make([]string, 0)
+
 	return runner.restoreInternal(args, data, flush, counters)
 }
 
@@ -402,6 +424,7 @@ func (runner *runner) restoreInternal(args []string, data []byte, flush FlushFla
 	if !flush {
 		args = append(args, "--noflush")
 	}
+
 	if counters {
 		args = append(args, "--counters")
 	}
@@ -414,10 +437,12 @@ func (runner *runner) restoreInternal(args []string, data []byte, flush FlushFla
 		if err != nil {
 			return err
 		}
+
 		trace.Step("Locks grabbed")
+
 		defer func(locker iptablesLocker) {
 			if err := locker.Close(); err != nil {
-				klog.ErrorS(err, "Failed to close iptables locks")
+				log.Errorf(context.Background(), "Failed to close iptables locks: %s", err)
 			}
 		}(locker)
 	}
@@ -425,17 +450,20 @@ func (runner *runner) restoreInternal(args []string, data []byte, flush FlushFla
 	// run the command and return the output or an error including the output and error
 	fullArgs := append(runner.restoreWaitFlag, args...) //nolint:gocritic
 	iptablesRestoreCmd := iptablesRestoreCommand(runner.protocol)
-	klog.V(4).InfoS("Running", "command", iptablesRestoreCmd, "arguments", fullArgs)
+	log.Debugf(context.Background(), "Running (command=%q arguments=%q)", iptablesRestoreCmd, fullArgs)
 	cmd := runner.exec.Command(iptablesRestoreCmd, fullArgs...)
 	cmd.SetStdin(bytes.NewBuffer(data))
+
 	b, err := cmd.CombinedOutput()
 	if err != nil {
 		pErr, ok := parseRestoreError(string(b))
 		if ok {
 			return pErr
 		}
+
 		return fmt.Errorf("%w: %s", err, b)
 	}
+
 	return nil
 }
 
@@ -443,6 +471,7 @@ func iptablesSaveCommand(protocol Protocol) string {
 	if protocol == ProtocolIPv6 {
 		return cmdIP6TablesSave
 	}
+
 	return cmdIPTablesSave
 }
 
@@ -450,6 +479,7 @@ func iptablesRestoreCommand(protocol Protocol) string {
 	if protocol == ProtocolIPv6 {
 		return cmdIP6TablesRestore
 	}
+
 	return cmdIPTablesRestore
 }
 
@@ -457,6 +487,7 @@ func iptablesCommand(protocol Protocol) string {
 	if protocol == ProtocolIPv6 {
 		return cmdIP6Tables
 	}
+
 	return cmdIPTables
 }
 
@@ -468,12 +499,14 @@ func (runner *runner) runContext(ctx context.Context, op operation, args []strin
 	iptablesCmd := iptablesCommand(runner.protocol)
 	fullArgs := append(runner.waitFlag, string(op)) //nolint:gocritic
 	fullArgs = append(fullArgs, args...)
-	klog.V(5).InfoS("Running", "command", iptablesCmd, "arguments", fullArgs)
+	log.Debugf(context.Background(), "Running (command=%q arguments=%+v)", iptablesCmd, fullArgs)
+
 	if ctx == nil {
 		return runner.exec.Command(iptablesCmd, fullArgs...).CombinedOutput()
 	}
-	return runner.exec.CommandContext(ctx, iptablesCmd, fullArgs...).CombinedOutput()
+
 	// Don't log err here - callers might not think it is an error.
+	return runner.exec.CommandContext(ctx, iptablesCmd, fullArgs...).CombinedOutput()
 }
 
 // Returns (bool, nil) if it was able to check the existence of the rule, or
@@ -482,6 +515,7 @@ func (runner *runner) checkRule(table Table, chain Chain, args ...string) (bool,
 	if runner.hasCheck {
 		return runner.checkRuleUsingCheck(makeFullArgs(table, chain, args...))
 	}
+
 	return runner.checkRuleWithoutCheck(table, chain, args...)
 }
 
@@ -496,7 +530,8 @@ func trimhex(s string) string {
 // of hack and half-measures.  We should nix this ASAP.
 func (runner *runner) checkRuleWithoutCheck(table Table, chain Chain, args ...string) (bool, error) {
 	iptablesSaveCmd := iptablesSaveCommand(runner.protocol)
-	klog.V(1).InfoS("Running", "command", iptablesSaveCmd, "table", string(table))
+	log.Debugf(context.Background(), "Running (command=%q table=%q)", iptablesSaveCmd, string(table))
+
 	out, err := runner.exec.Command(iptablesSaveCmd, "-t", string(table)).CombinedOutput()
 	if err != nil {
 		return false, fmt.Errorf("error checking rule: %w", err)
@@ -508,11 +543,13 @@ func (runner *runner) checkRuleWithoutCheck(table Table, chain Chain, args ...st
 	// in order to compare against iptables-save output (which will be split at whitespace boundary)
 	// e.g. a single arg('"this must be before the NodePort rules"') will be unquoted and unpacked into 7 args.
 	var argsCopy []string
+
 	for i := range args {
 		tmpField := strings.Trim(args[i], "\"")
 		tmpField = trimhex(tmpField)
 		argsCopy = append(argsCopy, strings.Fields(tmpField)...)
 	}
+
 	argset := sets.NewString(argsCopy...)
 
 	for _, line := range strings.Split(string(out), "\n") {
@@ -535,7 +572,8 @@ func (runner *runner) checkRuleWithoutCheck(table Table, chain Chain, args ...st
 		if sets.NewString(fields...).IsSuperset(argset) {
 			return true, nil
 		}
-		klog.V(5).InfoS("DBG: fields is not a superset of args", "fields", fields, "arguments", args)
+
+		log.Debugf(context.Background(), "fields is not a superset of args (fields=%+v arguments=%+v)", fields, args)
 	}
 
 	return false, nil
@@ -547,12 +585,14 @@ func (runner *runner) checkRuleUsingCheck(args []string) (bool, error) {
 	defer cancel()
 
 	out, err := runner.runContext(ctx, opCheckRule, args)
-	if ctx.Err() == context.DeadlineExceeded {
+	if errors.Is(ctx.Err(), context.DeadlineExceeded) {
 		return false, errors.New("timed out while checking rules")
 	}
+
 	if err == nil {
 		return true, nil
 	}
+
 	var ee utilexec.ExitError
 	if errors.As(err, &ee) {
 		// iptables uses exit(1) to indicate a failure of the operation,
@@ -561,6 +601,7 @@ func (runner *runner) checkRuleUsingCheck(args []string) (bool, error) {
 			return false, nil
 		}
 	}
+
 	return false, fmt.Errorf("error checking rule: %w: %s", err, out)
 }
 
@@ -577,10 +618,12 @@ func (runner *runner) Monitor(canary Chain, tables []Table, reloadFunc func(), i
 		_ = utilwait.PollImmediateUntil(interval, func() (bool, error) { //nolint:errcheck,staticcheck
 			for _, table := range tables {
 				if _, err := runner.EnsureChain(table, canary); err != nil {
-					klog.ErrorS(err, "Could not set up iptables canary", "table", table, "chain", canary)
+					log.Errorf(context.Background(), "Could not set up iptables canary: %s (table=%+v chain=%q)", err, table, canary)
+
 					return false, nil
 				}
 			}
+
 			return true, nil
 		}, stopCh)
 
@@ -589,10 +632,12 @@ func (runner *runner) Monitor(canary Chain, tables []Table, reloadFunc func(), i
 			if exists, err := runner.ChainExists(tables[0], canary); exists {
 				return false, nil
 			} else if isResourceError(err) {
-				klog.ErrorS(err, "Could not check for iptables canary", "table", tables[0], "chain", canary)
+				log.Errorf(context.Background(), "Could not check for iptables canary: %s (table=%q chain=%q)", err, tables[0], canary)
+
 				return false, nil
 			}
-			klog.V(2).InfoS("IPTables canary deleted", "table", tables[0], "chain", canary)
+
+			log.Debugf(context.Background(), "IPTables canary deleted (table=%q chain=%q)", tables[0], canary)
 			// Wait for the other canaries to be deleted too before returning
 			// so we don't start reloading too soon.
 			err := utilwait.PollImmediate(iptablesFlushPollTime, iptablesFlushTimeout, func() (bool, error) { //nolint:staticcheck
@@ -601,11 +646,13 @@ func (runner *runner) Monitor(canary Chain, tables []Table, reloadFunc func(), i
 						return false, nil
 					}
 				}
+
 				return true, nil
 			})
 			if err != nil {
-				klog.InfoS("Inconsistent iptables state detected")
+				log.Warnf(context.Background(), "Inconsistent iptables state detected: %s", err)
 			}
+
 			return true, nil
 		}, stopCh)
 		if err != nil {
@@ -613,10 +660,11 @@ func (runner *runner) Monitor(canary Chain, tables []Table, reloadFunc func(), i
 			for _, table := range tables {
 				_ = runner.DeleteChain(table, canary) //nolint:errcheck
 			}
+
 			return
 		}
 
-		klog.V(2).InfoS("Reloading after iptables flush")
+		log.Debugf(context.Background(), "Reloading after iptables flush")
 		reloadFunc()
 	}
 }
@@ -632,6 +680,7 @@ func (runner *runner) ChainExists(table Table, chain Chain) (bool, error) {
 	defer trace.LogIfLong(2 * time.Second)
 
 	_, err := runner.run(opListChain, fullArgs)
+
 	return err == nil, err
 }
 
@@ -656,15 +705,19 @@ const iptablesVersionPattern = `v([0-9]+(\.[0-9]+)+)`
 func getIPTablesVersion(exec utilexec.Interface, protocol Protocol) (*utilversion.Version, error) {
 	// this doesn't access mutable state so we don't need to use the interface / runner
 	iptablesCmd := iptablesCommand(protocol)
+
 	b, err := exec.Command(iptablesCmd, "--version").CombinedOutput()
 	if err != nil {
 		return nil, err
 	}
+
 	versionMatcher := regexp.MustCompile(iptablesVersionPattern) //nolint:gocritic
+
 	match := versionMatcher.FindStringSubmatch(string(b))
 	if match == nil {
 		return nil, fmt.Errorf("no iptables version found in string: %s", b)
 	}
+
 	version, err := utilversion.ParseGeneric(match[1])
 	if err != nil {
 		return nil, fmt.Errorf("iptables version %q is not a valid version string: %w", match[1], err)
@@ -697,13 +750,17 @@ func getIPTablesRestoreWaitFlag(version *utilversion.Version, exec utilexec.Inte
 	// --version, assume it also supports --wait
 	vstring, err := getIPTablesRestoreVersionString(exec, protocol)
 	if err != nil || vstring == "" {
-		klog.V(3).InfoS("Couldn't get iptables-restore version; assuming it doesn't support --wait")
+		log.Warnf(context.Background(), "Couldn't get iptables-restore version; assuming it doesn't support --wait: %q", err)
+
 		return nil
 	}
+
 	if _, err := utilversion.ParseGeneric(vstring); err != nil {
-		klog.V(3).InfoS("Couldn't parse iptables-restore version; assuming it doesn't support --wait")
+		log.Warnf(context.Background(), "Couldn't parse iptables-restore version; assuming it doesn't support --wait: %q", err)
+
 		return nil
 	}
+
 	return []string{WaitString}
 }
 
@@ -711,22 +768,25 @@ func getIPTablesRestoreWaitFlag(version *utilversion.Version, exec utilexec.Inte
 // in the form "X.X.X".
 func getIPTablesRestoreVersionString(exec utilexec.Interface, protocol Protocol) (string, error) {
 	// this doesn't access mutable state so we don't need to use the interface / runner
-
 	// iptables-restore hasn't always had --version, and worse complains
 	// about unrecognized commands but doesn't exit when it gets them.
 	// Work around that by setting stdin to nothing so it exits immediately.
 	iptablesRestoreCmd := iptablesRestoreCommand(protocol)
 	cmd := exec.Command(iptablesRestoreCmd, "--version")
 	cmd.SetStdin(bytes.NewReader([]byte{}))
+
 	b, err := cmd.CombinedOutput()
 	if err != nil {
 		return "", err
 	}
+
 	versionMatcher := regexp.MustCompile(iptablesVersionPattern) //nolint:gocritic
+
 	match := versionMatcher.FindStringSubmatch(string(b))
 	if match == nil {
 		return "", fmt.Errorf("no iptables version found in string: %s", b)
 	}
+
 	return match[1], nil
 }
 
@@ -778,6 +838,7 @@ func IsNotFoundError(err error) bool {
 			return true
 		}
 	}
+
 	return false
 }
 
@@ -791,6 +852,7 @@ func isResourceError(err error) bool {
 	if errors.As(err, &ee) {
 		return ee.ExitStatus() == iptablesStatusResourceProblem
 	}
+
 	return false
 }
 
@@ -837,15 +899,19 @@ func parseRestoreError(str string) (ParseError, bool) {
 	if len(errs) != 2 {
 		return nil, false
 	}
+
 	cmd := errs[0]
+
 	matches := regexpParseError.FindStringSubmatch(errs[1])
 	if len(matches) != 2 {
 		return nil, false
 	}
+
 	line, errMsg := strconv.Atoi(matches[1])
 	if errMsg != nil {
 		return nil, false
 	}
+
 	return parseError{cmd: cmd, line: line}, true
 }
 
@@ -856,15 +922,18 @@ func ExtractLines(lines []byte, line, count int) []LineData {
 	if line < 1 {
 		return nil
 	}
+
 	start := line - count
 	if start <= 0 {
 		start = 1
 	}
+
 	end := line + count + 1
 
 	offset := 1
 	scanner := bufio.NewScanner(bytes.NewBuffer(lines))
 	extractLines := make([]LineData, 0, count*2)
+
 	for scanner.Scan() {
 		if offset >= start && offset < end {
 			extractLines = append(extractLines, LineData{
@@ -872,10 +941,13 @@ func ExtractLines(lines []byte, line, count int) []LineData {
 				Data: scanner.Text(),
 			})
 		}
+
 		if offset == end {
 			break
 		}
+
 		offset++
 	}
+
 	return extractLines
 }
