@@ -7,9 +7,9 @@ import (
 	"os"
 	"time"
 
-	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+	"go.uber.org/mock/gomock"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 
 	"github.com/cri-o/cri-o/internal/lib"
@@ -38,6 +38,11 @@ var _ = t.Describe("ContainerServer", func() {
 			config.HooksDir = []string{}
 			// so we have permission to make a directory within it
 			config.ContainerAttachSocketDir = t.MustTempDir("crio")
+			// Simulate a clean shutdown. Otherwise, when running tests as root on
+			// a system where cri-o is already installed, we hit non-mocked functions
+			// in lib.New internal/lib/container_server.go in if condition
+			// `if config.InternalRepair && ShutdownWasUnclean(config)`.
+			config.CleanShutdownFile = t.MustTempFile("clean.shutdown")
 
 			// Specify mocks
 			gomock.InOrder(
