@@ -3,8 +3,8 @@ package mockutils
 import (
 	"fmt"
 
-	"github.com/golang/mock/gomock"
 	"github.com/onsi/ginkgo/v2"
+	"go.uber.org/mock/gomock"
 )
 
 type MockSequence struct {
@@ -12,10 +12,10 @@ type MockSequence struct {
 }
 
 // like gomock.InOrder, but can be nested.
-func InOrder(calls ...interface{}) MockSequence {
+func InOrder(calls ...any) MockSequence {
 	var first, last *gomock.Call
 	// This implementation does a few more assignments and checks than strictly necessary, but it is O(N) and reasonably easy to read, so, whatever.
-	for i := range len(calls) {
+	for i := range calls {
 		var elem MockSequence
 		switch e := calls[i].(type) {
 		case MockSequence:
@@ -29,12 +29,15 @@ func InOrder(calls ...interface{}) MockSequence {
 		if elem.first == nil {
 			continue
 		}
+
 		if first == nil {
 			first = elem.first
 		} else if last != nil {
 			elem.first.After(last)
 		}
+
 		last = elem.last
 	}
+
 	return MockSequence{first, last}
 }
