@@ -2,6 +2,7 @@ package oci
 
 import (
 	"bytes"
+	"context"
 	"encoding/base64"
 	"encoding/json"
 	"errors"
@@ -34,7 +35,6 @@ import (
 	katavolume "github.com/kata-containers/kata-containers/src/runtime/virtcontainers/types"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/sirupsen/logrus"
-	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
 	anypb "google.golang.org/protobuf/types/known/anypb"
 	"k8s.io/client-go/tools/remotecommand"
@@ -109,13 +109,12 @@ func newRuntimeVM(handler *config.RuntimeHandler, exitsPath string) RuntimeImpl
 
 func addVolumeMountsToCreateRequest(ctx context.Context, request *task.CreateTaskRequest, c *Container) error {
 	// To make the kata agent pull the image{"volume_type":"image_guest_pull","source":"quay.io/kata-containers/confidential-containers:unsigned","fs_type":"overlayfs","image_pull":{"metadata":{}}}
-
-	imageSource := c.Spec().Annotations[annotations.ImageName]
-	log.Infof(ctx, "Adding mount info to pull image %s", imageSource)
+	someNameOfTheImageSource := c.Spec().Annotations[annotations.SomeNameOfTheImage]
+	log.Infof(ctx, "Adding mount info to pull image %s", someNameOfTheImageSource)
 
 	volume := &katavolume.KataVirtualVolume{
 		VolumeType: katavolume.KataVirtualVolumeImageGuestPullType,
-		Source:     imageSource,
+		Source:     someNameOfTheImageSource,
 		FSType:     "overlay_fs",
 		ImagePull:  &katavolume.ImagePullVolume{Metadata: c.Spec().Annotations},
 	}
@@ -139,6 +138,7 @@ func addVolumeMountsToCreateRequest(ctx context.Context, request *task.CreateTas
 		Type:    "fuse.nydus-overlayfs",
 		Options: []string{opt},
 	})
+
 	return nil
 }
 
@@ -156,6 +156,7 @@ func (r *runtimeVM) CreateContainer(ctx context.Context, c *Container, cgroupPar
 	// the runtime_config_path to an arbitrary location.  Also, lets
 	// fail early if something goes wrong.
 	var opts *anypb.Any = nil
+
 	if r.configPath != "" {
 		runtimeOptions := &runtimeoptions.Options{
 			ConfigPath: r.configPath,
@@ -182,9 +183,11 @@ func (r *runtimeVM) CreateContainer(ctx context.Context, c *Container, cgroupPar
 	defer func() {
 		if retErr != nil {
 			log.Warnf(ctx, "Cleaning up container %s: %v", c.ID(), retErr)
+
 			if cleanupErr := r.deleteContainer(c, true); cleanupErr != nil {
 				log.Infof(ctx, "DeleteContainer failed for container %s: %v", c.ID(), cleanupErr)
 			}
+
 			if err := os.Remove(c.logPath); err != nil {
 				log.Warnf(ctx, "Failed to remove log path %s after failing to create container: %v", c.logPath, err)
 			}
@@ -210,6 +213,7 @@ func (r *runtimeVM) CreateContainer(ctx context.Context, c *Container, cgroupPar
 	}
 
 	createdCh := make(chan error)
+
 	go func() {
 		// Create the container
 		if resp, err := r.task.Create(r.ctx, request); err != nil {
@@ -230,7 +234,9 @@ func (r *runtimeVM) CreateContainer(ctx context.Context, c *Container, cgroupPar
 		if err := r.remove(c.ID(), ""); err != nil {
 			return err
 		}
+
 		<-createdCh
+
 		return fmt.Errorf("CreateContainer timeout (%v)", ContainerCreateTimeout)
 	}
 
@@ -243,10 +249,12 @@ func (r *runtimeVM) startRuntimeDaemon(ctx context.Context, c *Container) error
 
 	// Prepare the command to run
 	args := []string{"-id", c.ID()}
+
 	switch logrus.GetLevel() {
 	case logrus.DebugLevel, logrus.TraceLevel:
 		args = append(args, "-debug")
 	}
+
 	args = append(args, "start")
 
 	r.ctx = namespaces.WithNamespace(r.ctx, namespaces.Default)
@@ -276,6 +284,7 @@ func (r *runtimeVM) startRuntimeDaemon(ctx context.Context, c *Container) error
 	// to CRI-O's output.
 	go func() {
 		defer f.Close()
+
 		if _, err := io.Copy(os.Stderr, f); err != nil {
 			log.Errorf(ctx, "Copy shim log: %v", err)
 		}
@@ -318,6 +327,7 @@ func (r *runtimeVM) StartContainer(ctx context.Context, c *Container) error {
 	if err := r.start(c.ID(), ""); err != nil {
 		return err
 	}
+
 	c.state.Started = time.Now()
 
 	// Spawn a goroutine waiting for the container to terminate. Once it
@@ -330,6 +340,7 @@ func (r *runtimeVM) StartContainer(ctx context.Context, c *Container) error {
 			if fileErr := os.WriteFile(path, []byte("Exited"), 0o644); fileErr != nil {
 				log.Warnf(ctx, "Unable to write exit file %v", fileErr)
 			}
+
 			if err1 := r.updateContainerStatus(ctx, c); err1 != nil {
 				log.Warnf(ctx, "Error updating container status %v", err1)
 			}
@@ -350,6 +361,7 @@ func (r *runtimeVM) ExecContainer(ctx context.Context, c *Container, cmd []strin
 	if err != nil {
 		return err
 	}
+
 	if exitCode != 0 {
 		return &utilexec.CodeExitError{
 			Err:  fmt.Errorf("executing command %v, exit code %d", cmd, exitCode),
@@ -379,6 +391,7 @@ func (r *runtimeVM) ExecSyncContainer(ctx context.Context, c *Container, command
 	defer log.Debugf(ctx, "RuntimeVM.ExecSyncContainer() end")
 
 	var stdoutBuf, stderrBuf bytes.Buffer
+
 	stdout := &writeCloserWrapper{limitWriter(&stdoutBuf, maxExecSyncSize)}
 	stderr := &writeCloserWrapper{limitWriter(&stderrBuf, maxExecSyncSize)}
 
@@ -422,17 +435,22 @@ func (l *limitedWriter) Write(p []byte) (n int, err error) {
 	if l.N <= 0 {
 		return 0, io.ErrShortWrite
 	}
+
 	truncated := false
+
 	if int64(len(p)) > l.N {
 		p = p[0:l.N]
 		truncated = true
 	}
+
 	n, err = l.W.Write(p)
 	l.N -= int64(n)
+
 	if err == nil && truncated {
 		err = io.ErrShortWrite
 	}
-	return
+
+	return n, err
 }
 
 func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd []string, timeout int64, stdin io.Reader, stdout, stderr io.WriteCloser, tty bool, resizeChan <-chan remotecommand.TerminalSize) (exitCode int32, retErr error) {
@@ -458,6 +476,7 @@ func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd [
 
 	// chan to notify that can call runtime's CloseIO API
 	closeIOChan := make(chan bool)
+
 	defer func() {
 		if closeIOChan != nil {
 			close(closeIOChan)
@@ -474,6 +493,7 @@ func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd [
 			if closeIOChan != nil {
 				<-closeIOChan
 			}
+
 			return r.closeIO(c.ID(), execID)
 		},
 	})
@@ -483,7 +503,7 @@ func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd [
 	pSpec := *c.Spec().Process
 	pSpec.Args = cmd
 
-	any, err := typeurl.MarshalAny(&pSpec)
+	anyType, err := typeurl.MarshalAny(&pSpec)
 	if err != nil {
 		return execError, errdefs.FromGRPC(err)
 	}
@@ -495,7 +515,7 @@ func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd [
 		Stdout:   execIO.Config().Stdout,
 		Stderr:   execIO.Config().Stderr,
 		Terminal: execIO.Config().Terminal,
-		Spec:     protobuf.FromAny(any),
+		Spec:     protobuf.FromAny(anyType),
 	}
 
 	// Create the "exec" process
@@ -542,6 +562,7 @@ func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd [
 	}
 
 	execCh := make(chan error)
+
 	go func() {
 		// Wait for the process to terminate
 		exitCode, err = r.wait(c.ID(), execID)
@@ -558,12 +579,14 @@ func (r *runtimeVM) execContainerCommon(ctx context.Context, c *Container, cmd [
 			if killErr := r.kill(c.ID(), execID, syscall.SIGKILL, false); killErr != nil {
 				return execError, killErr
 			}
+
 			return execError, err
 		}
 	case <-timeoutCh:
 		if killErr := r.kill(c.ID(), execID, syscall.SIGKILL, false); killErr != nil {
 			return execError, killErr
 		}
+
 		<-execCh
 		// do not make an error for timeout: report it with a specific error code
 		return execTimeout, nil
@@ -589,14 +612,14 @@ func (r *runtimeVM) UpdateContainer(ctx context.Context, c *Container, res *rspe
 	defer c.opLock.Unlock()
 
 	// Convert resources into protobuf Any type
-	any, err := typeurl.MarshalAny(res)
+	anyType, err := typeurl.MarshalAny(res)
 	if err != nil {
 		return err
 	}
 
 	if _, err := r.task.Update(r.ctx, &task.UpdateTaskRequest{
 		ID:        c.ID(),
-		Resources: protobuf.FromAny(any),
+		Resources: protobuf.FromAny(anyType),
 	}); err != nil {
 		return errdefs.FromGRPC(err)
 	}
@@ -613,6 +636,7 @@ func (r *runtimeVM) StopContainer(ctx context.Context, c *Container, timeout int
 		if errors.Is(err, ErrContainerStopped) {
 			err = nil
 		}
+
 		return err
 	}
 
@@ -625,6 +649,7 @@ func (r *runtimeVM) StopContainer(ctx context.Context, c *Container, timeout int
 	defer cancel()
 
 	stopCh := make(chan error)
+
 	go func() {
 		// errdefs.ErrNotFound actually comes from a closed connection, which is expected
 		// when stopping the container, with the agent and the VM going off. In such case.
@@ -650,8 +675,10 @@ func (r *runtimeVM) StopContainer(ctx context.Context, c *Container, timeout int
 		err := r.waitCtrTerminate(sig, stopCh, timeoutDuration)
 		if err == nil {
 			c.state.Finished = time.Now()
+
 			return nil
 		}
+
 		log.Warnf(ctx, "%v", err)
 	}
 
@@ -663,10 +690,12 @@ func (r *runtimeVM) StopContainer(ctx context.Context, c *Container, timeout int
 
 	if err := r.waitCtrTerminate(sig, stopCh, killContainerTimeout); err != nil {
 		log.Errorf(ctx, "%v", err)
+
 		return err
 	}
 
 	c.state.Finished = time.Now()
+
 	return nil
 }
 
@@ -681,13 +710,16 @@ func (r *runtimeVM) shouldBeStopped(ctx context.Context, c *Container) error {
 		return ErrContainerStopped
 	case ContainerStatePaused:
 		log.Warnf(ctx, "Cannot stop paused container %s", c.ID())
+
 		if err := r.UnpauseContainer(ctx, c); err != nil {
 			return fmt.Errorf("failed to stop container %s: %w", c.Name(), err)
 		}
+
 		if err := r.UpdateContainerStatus(ctx, c); err != nil {
 			return fmt.Errorf("failed to update container status %s: %w", c.Name(), err)
 		}
 	}
+
 	return nil
 }
 
@@ -708,6 +740,7 @@ func (r *runtimeVM) DeleteContainer(ctx context.Context, c *Container) error {
 	// Lock the container
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
+
 	if c.state.OOMKilled {
 		// Collect metric by container name
 		metrics.Instance().MetricContainersOOMCountTotalDelete(c.Name())
@@ -723,6 +756,7 @@ func (r *runtimeVM) deleteContainer(c *Container, force bool) error {
 	r.Lock()
 	cInfo, ok := r.ctrs[c.ID()]
 	r.Unlock()
+
 	if !ok && !force {
 		return errors.New("could not retrieve container information")
 	}
@@ -770,23 +804,29 @@ func (r *runtimeVM) updateContainerStatus(ctx context.Context, c *Container) err
 	// And then connect to the existing gRPC server with this address.
 	if r.task == nil {
 		addressPath := filepath.Join(c.BundlePath(), "address")
+
 		data, err := os.ReadFile(addressPath)
 		if err != nil {
 			// If the container is actually removed, this error is expected and should be ignored.
 			// In this case, the container's status should be "Stopped".
 			if c.state.Status == ContainerStateStopped {
 				log.Debugf(ctx, "Skipping status update for: %+v", c.state)
+
 				return nil
 			}
 
 			log.Warnf(ctx, "Failed to read shim address: %v", err)
+
 			return errors.New("runtime not correctly setup")
 		}
+
 		address := strings.TrimSpace(string(data))
+
 		conn, err := client.Connect(address, client.AnonDialer)
 		if err != nil {
 			return err
 		}
+
 		options := ttrpc.WithOnClose(func() { conn.Close() })
 		cl := ttrpc.NewClient(conn, options)
 		r.client = cl
@@ -800,6 +840,7 @@ func (r *runtimeVM) updateContainerStatus(ctx context.Context, c *Container) err
 		if !errors.Is(err, ttrpc.ErrClosed) {
 			return errdefs.FromGRPC(err)
 		}
+
 		return errdefs.ErrNotFound
 	}
 
@@ -808,6 +849,7 @@ func (r *runtimeVM) updateContainerStatus(ctx context.Context, c *Container) err
 	}
 
 	status := c.state.Status
+
 	switch response.GetStatus() {
 	case tasktypes.Status_CREATED:
 		status = ContainerStateCreated
@@ -837,16 +879,20 @@ func (r *runtimeVM) updateContainerStatus(ctx context.Context, c *Container) err
 			metrics.Instance().MetricContainersOOMCountTotalInc(c.Name())
 		}
 	}
+
 	return nil
 }
 
 func (r *runtimeVM) restoreContainerIO(ctx context.Context, c *Container, state *task.StateResponse) error {
 	r.Lock()
+
 	_, ok := r.ctrs[c.ID()]
 	if ok {
 		r.Unlock()
+
 		return nil
 	}
+
 	r.Unlock()
 
 	cioCfg := ctrio.Config{
@@ -862,12 +908,15 @@ func (r *runtimeVM) restoreContainerIO(ctx context.Context, c *Container, state
 	if cioCfg.Stdin != "" {
 		iofiles = append(iofiles, cioCfg.Stdin)
 	}
+
 	if cioCfg.Stdout != "" {
 		iofiles = append(iofiles, cioCfg.Stdout)
 	}
+
 	if cioCfg.Stderr != "" {
 		iofiles = append(iofiles, cioCfg.Stderr)
 	}
+
 	closer := func() error {
 		for _, f := range iofiles {
 			if err := os.Remove(f); err != nil {
@@ -878,9 +927,11 @@ func (r *runtimeVM) restoreContainerIO(ctx context.Context, c *Container, state
 		for _, f := range iofiles {
 			_ = os.Remove(filepath.Dir(f))
 		}
+
 		return nil
 	}
 	_, err := r.createContainerIO(ctx, c, cio.WithFIFOs(ctrio.NewFIFOSet(cioCfg, closer)))
+
 	return err
 }
 
@@ -897,27 +948,11 @@ func (r *runtimeVM) createContainerIO(ctx context.Context, c *Container, cioOpts
 		}
 	}()
 
-	f, err := os.OpenFile(c.LogPath(), os.O_WRONLY|os.O_APPEND|os.O_CREATE, 0o600)
+	stdout, stderr, err := r.createContainerLoggers(ctx, c.LogPath())
 	if err != nil {
 		return nil, err
 	}
 
-	var stdoutCh, stderrCh <-chan struct{}
-	wc := cioutil.NewSerialWriteCloser(f)
-	stdout, stdoutCh := cio.NewCRILogger(c.LogPath(), wc, cio.Stdout, -1)
-	stderr, stderrCh := cio.NewCRILogger(c.LogPath(), wc, cio.Stderr, -1)
-
-	go func() {
-		if stdoutCh != nil {
-			<-stdoutCh
-		}
-		if stderrCh != nil {
-			<-stderrCh
-		}
-		log.Debugf(ctx, "Finish redirecting log file %q, closing it", c.LogPath())
-		f.Close()
-	}()
-
 	containerIO.AddOutput(c.LogPath(), stdout, stderr)
 	containerIO.Pipe()
 
@@ -930,6 +965,35 @@ func (r *runtimeVM) createContainerIO(ctx context.Context, c *Container, cioOpts
 	return containerIO, nil
 }
 
+// createContainerLoggers creates container loggers and return write closer for stdout and stderr.
+func (r *runtimeVM) createContainerLoggers(ctx context.Context, logPath string) (stdout, stderr io.WriteCloser, err error) {
+	f, err := os.OpenFile(logPath, os.O_WRONLY|os.O_APPEND|os.O_CREATE, 0o600)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	var stdoutCh, stderrCh <-chan struct{}
+
+	wc := cioutil.NewSerialWriteCloser(f)
+	stdout, stdoutCh = cio.NewCRILogger(logPath, wc, cio.Stdout, -1)
+	stderr, stderrCh = cio.NewCRILogger(logPath, wc, cio.Stderr, -1)
+
+	go func() {
+		if stdoutCh != nil {
+			<-stdoutCh
+		}
+
+		if stderrCh != nil {
+			<-stderrCh
+		}
+
+		log.Debugf(ctx, "Finish redirecting log file %q, closing it", logPath)
+		f.Close()
+	}()
+
+	return stdout, stderr, nil
+}
+
 // PauseContainer pauses a container.
 func (r *runtimeVM) PauseContainer(ctx context.Context, c *Container) error {
 	log.Debugf(ctx, "RuntimeVM.PauseContainer() start")
@@ -981,6 +1045,7 @@ func (r *runtimeVM) ContainerStats(ctx context.Context, c *Container, _ string)
 	if err != nil {
 		return nil, errdefs.FromGRPC(err)
 	}
+
 	if resp == nil {
 		return nil, errors.New("could not retrieve container stats")
 	}
@@ -1002,6 +1067,7 @@ func (r *runtimeVM) ContainerStats(ctx context.Context, c *Container, _ string)
 			return metricsV2ToCgroupStats(ctx, m), nil
 		}
 	}
+
 	return nil, fmt.Errorf("unknown stats type %T", stats)
 }
 
@@ -1011,8 +1077,10 @@ func metricsV1ToCgroupStats(ctx context.Context, m *cgroupsV1.Metrics) *cgmgr.Cg
 		memUsage        uint64
 		workingSetBytes uint64
 	)
+
 	memUsage = m.Memory.Usage.Usage
 	memLimit = cgmgr.MemLimitGivenSystem(m.Memory.Usage.Limit)
+
 	if memUsage > m.Memory.TotalInactiveFile {
 		workingSetBytes = memUsage - m.Memory.TotalInactiveFile
 	} else {
@@ -1021,6 +1089,15 @@ func metricsV1ToCgroupStats(ctx context.Context, m *cgroupsV1.Metrics) *cgmgr.Cg
 			m.Memory.TotalInactiveFile, memUsage,
 		)
 	}
+
+	hugetlbStats := map[string]cgmgr.HugetlbStats{}
+	for _, hugetlb := range m.Hugetlb {
+		hugetlbStats[hugetlb.Pagesize] = cgmgr.HugetlbStats{
+			Usage: hugetlb.Usage,
+			Max:   hugetlb.Max,
+		}
+	}
+
 	return &cgmgr.CgroupStats{
 		Memory: &cgmgr.MemoryStats{
 			Usage:           memUsage,
@@ -1048,6 +1125,7 @@ func metricsV1ToCgroupStats(ctx context.Context, m *cgroupsV1.Metrics) *cgmgr.Cg
 			ThrottlingActivePeriods: m.CPU.Throttling.Periods,
 			ThrottledPeriods:        m.CPU.Throttling.ThrottledPeriods,
 		},
+		Hugetlb: hugetlbStats,
 		Pid: &cgmgr.PidsStats{
 			Current: m.Pids.Current,
 			Limit:   m.Pids.Limit,
@@ -1062,9 +1140,11 @@ func metricsV2ToCgroupStats(ctx context.Context, m *cgroupsV2.Metrics) *cgmgr.Cg
 		memUsage        uint64
 		workingSetBytes uint64
 	)
+
 	if m != nil {
 		memUsage = m.Memory.Usage
 		memLimit = cgmgr.MemLimitGivenSystem(m.Memory.UsageLimit)
+
 		if memUsage > m.Memory.InactiveFile {
 			workingSetBytes = memUsage - m.Memory.InactiveFile
 		} else {
@@ -1074,6 +1154,15 @@ func metricsV2ToCgroupStats(ctx context.Context, m *cgroupsV2.Metrics) *cgmgr.Cg
 			)
 		}
 	}
+
+	hugetlbStats := map[string]cgmgr.HugetlbStats{}
+	for _, hugetlb := range m.Hugetlb {
+		hugetlbStats[hugetlb.Pagesize] = cgmgr.HugetlbStats{
+			Usage: hugetlb.Current,
+			Max:   hugetlb.Max,
+		}
+	}
+
 	return &cgmgr.CgroupStats{
 		Memory: &cgmgr.MemoryStats{
 			Usage:           memUsage,
@@ -1101,6 +1190,7 @@ func metricsV2ToCgroupStats(ctx context.Context, m *cgroupsV2.Metrics) *cgmgr.Cg
 			ThrottledPeriods:        m.CPU.NrThrottled,
 			ThrottledTime:           m.CPU.ThrottledUsec * 1000,
 		},
+		Hugetlb: hugetlbStats,
 		Pid: &cgmgr.PidsStats{
 			Current: m.Pids.Current,
 			Limit:   m.Pids.Limit,
@@ -1138,6 +1228,7 @@ func (r *runtimeVM) AttachContainer(ctx context.Context, c *Container, inputStre
 	r.Lock()
 	cInfo, ok := r.ctrs[c.ID()]
 	r.Unlock()
+
 	if !ok {
 		return errors.New("could not retrieve container information")
 	}
@@ -1154,6 +1245,7 @@ func (r *runtimeVM) AttachContainer(ctx context.Context, c *Container, inputStre
 	}
 
 	cInfo.cio.Attach(opts)
+
 	return nil
 }
 
@@ -1170,6 +1262,29 @@ func (r *runtimeVM) ReopenContainerLog(ctx context.Context, c *Container) error
 	log.Debugf(ctx, "RuntimeVM.ReopenContainerLog() start")
 	defer log.Debugf(ctx, "RuntimeVM.ReopenContainerLog() end")
 
+	r.Lock()
+	cInfo, ok := r.ctrs[c.ID()]
+	r.Unlock()
+
+	if !ok {
+		return errors.New("could not retrieve container information")
+	}
+
+	// Create new container logger and replace the existing ones.
+	stdoutWC, stderrWC, err := r.createContainerLoggers(ctx, c.LogPath())
+	if err != nil {
+		return err
+	}
+
+	oldStdoutWC, oldStderrWC := cInfo.cio.AddOutput(c.LogPath(), stdoutWC, stderrWC)
+	if oldStdoutWC != nil {
+		oldStdoutWC.Close()
+	}
+
+	if oldStderrWC != nil {
+		oldStderrWC.Close()
+	}
+
 	return nil
 }
 
@@ -1193,6 +1308,7 @@ func (r *runtimeVM) wait(ctrID, execID string) (int32, error) {
 		if !errors.Is(err, ttrpc.ErrClosed) {
 			return -1, errdefs.FromGRPC(err)
 		}
+
 		return -1, errdefs.ErrNotFound
 	}
 
@@ -1271,7 +1387,26 @@ func EncodeKataVirtualVolumeToBase64(ctx context.Context, volume *katavolume.Kat
 	if err != nil {
 		return "", fmt.Errorf("marshal KataVirtualVolume object; error=%w", err)
 	}
+
 	log.Infof(ctx, "Encode kata volume %v", validKataVirtualVolumeJSON)
 	option := base64.StdEncoding.EncodeToString(validKataVirtualVolumeJSON)
+
 	return option, nil
 }
+
+func (r *runtimeVM) IsContainerAlive(c *Container) bool {
+	return r.kill(c.ID(), "", 0, false) == nil
+}
+
+func (r *runtimeVM) ProbeMonitor(ctx context.Context, c *Container) error {
+	// Not implemented
+	return nil
+}
+
+func (r *runtimeVM) ServeExecContainer(context.Context, *Container, []string, bool, bool, bool, bool) (string, error) {
+	return "", nil
+}
+
+func (r *runtimeVM) ServeAttachContainer(context.Context, *Container, bool, bool, bool) (string, error) {
+	return "", nil
+}
