@@ -2,6 +2,8 @@ package runtimehandlerhooks
 
 import (
 	"context"
+	"errors"
+	"fmt"
 	"os"
 	"path/filepath"
 	"strconv"
@@ -14,6 +16,7 @@ import (
 	"github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
+	"k8s.io/utils/cpuset"
 
 	"github.com/cri-o/cri-o/internal/hostport"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
@@ -37,6 +40,66 @@ const (
 	governorUserspace    = "userspace"
 )
 
+type mockServiceManager struct {
+	isServiceEnabled map[string]bool
+	restartService   map[string]error
+	history          []string
+}
+
+func (m *mockServiceManager) IsServiceEnabled(serviceName string) bool {
+	m.history = append(m.history, "systemctl is-enabled "+serviceName)
+	if m.isServiceEnabled == nil {
+		return false
+	}
+
+	return m.isServiceEnabled[serviceName]
+}
+
+func (m *mockServiceManager) RestartService(serviceName string) error {
+	m.history = append(m.history, "systemctl restart "+serviceName)
+	if m.restartService == nil {
+		return errors.New("service not found")
+	}
+
+	if _, ok := m.restartService[serviceName]; !ok {
+		return errors.New("service not found")
+	}
+
+	return m.restartService[serviceName]
+}
+
+type mockCommandRunner struct {
+	lookPath map[string]struct {
+		path string
+		err  error
+	}
+	history []string
+}
+
+func (m *mockCommandRunner) LookPath(file string) (string, error) {
+	m.history = append(m.history, "which "+file)
+	if m.lookPath == nil {
+		return "", errors.New("path not found")
+	}
+
+	if _, ok := m.lookPath[file]; !ok {
+		return "", errors.New("path not found")
+	}
+
+	return m.lookPath[file].path, m.lookPath[file].err
+}
+
+func (m *mockCommandRunner) RunCommand(name string, env []string, arg ...string) error {
+	m.history = append(m.history, fmt.Sprintf(
+		"%s %s %s",
+		strings.Join(env, " "),
+		name,
+		strings.Join(arg, " "),
+	))
+
+	return nil
+}
+
 // The actual test suite.
 var _ = Describe("high_performance_hooks", func() {
 	container, err := oci.NewContainer("containerID", "", "", "",
@@ -48,6 +111,34 @@ var _ = Describe("high_performance_hooks", func() {
 
 	var flags, bannedCPUFlags string
 
+	baseSandboxBuilder := func() sandbox.Builder {
+		sbox := sandbox.NewBuilder()
+		createdAt := time.Now()
+		sbox.SetCreatedAt(createdAt)
+		sbox.SetID("sandboxID")
+		sbox.SetName("sandboxName")
+		sbox.SetLogDir("test")
+		sbox.SetShmPath("test")
+		sbox.SetNamespace("")
+		sbox.SetKubeName("")
+		sbox.SetMountLabel("test")
+		sbox.SetProcessLabel("test")
+		sbox.SetCgroupParent("")
+		sbox.SetRuntimeHandler("")
+		sbox.SetResolvPath("")
+		sbox.SetHostname("")
+		sbox.SetPortMappings([]*hostport.PortMapping{})
+		sbox.SetHostNetwork(false)
+		sbox.SetUsernsMode("")
+		sbox.SetPodLinuxOverhead(nil)
+		sbox.SetPodLinuxResources(nil)
+		sbox.SetPrivileged(false)
+		sbox.SetHostNetwork(false)
+		sbox.SetCreatedAt(createdAt)
+
+		return sbox
+	}
+
 	BeforeEach(func() {
 		err := os.MkdirAll(fixturesDir, os.ModePerm)
 		Expect(err).ToNot(HaveOccurred())
@@ -66,7 +157,7 @@ var _ = Describe("high_performance_hooks", func() {
 				irqBalanceConfigFile: irqBalanceConfigFile,
 				irqSMPAffinityFile:   irqSmpAffinityFile,
 			}
-			err := h.setIRQLoadBalancing(context.TODO(), container, enabled)
+			err := h.setIRQLoadBalancing(context.TODO(), container, cpuset.CPUSet{}, enabled)
 			Expect(err).ToNot(HaveOccurred())
 
 			content, err := os.ReadFile(irqSmpAffinityFile)
@@ -123,7 +214,7 @@ var _ = Describe("high_performance_hooks", func() {
 				irqBalanceConfigFile: irqBalanceConfigFile,
 				irqSMPAffinityFile:   irqSmpAffinityFile,
 			}
-			err = h.setIRQLoadBalancing(context.TODO(), container, enabled)
+			err = h.setIRQLoadBalancing(context.TODO(), container, cpuset.CPUSet{}, enabled)
 			Expect(err).ToNot(HaveOccurred())
 
 			content, err := os.ReadFile(irqSmpAffinityFile)
@@ -187,6 +278,211 @@ var _ = Describe("high_performance_hooks", func() {
 		})
 	})
 
+	Describe("setIRQLoadBalancing with housekeeping CPUs", func() {
+		irqSmpAffinityFile := filepath.Join(fixturesDir, "irq_smp_affinity")
+		irqBalanceConfigFile := filepath.Join(fixturesDir, "irqbalance")
+		sysCPUDir := filepath.Join(fixturesDir, "cpus")
+
+		createSysCPUThreadSiblingsDir := func(testCPUDir string, numCPUs, topology int) {
+			err := os.MkdirAll(testCPUDir, os.ModePerm)
+			Expect(err).ToNot(HaveOccurred())
+
+			Expect(numCPUs%topology).To(Equal(0), "num cpus and topology mismatch")
+
+			// Create CPU directories and topology files for CPUs.
+			for cpu := 0; cpu < numCPUs; cpu += topology {
+				// Create thread siblings based on hyperthreading simulation.
+				// E.g., with topology == 2, CPUs 0,1 are siblings; 2,3 are siblings; 4,5 are siblings; 6,7 are siblings.
+				siblings := []int{}
+				for sibling := cpu; sibling < cpu+topology; sibling++ {
+					siblings = append(siblings, sibling)
+				}
+				siblingsSet := cpuset.New(siblings...)
+
+				for sibling := cpu; sibling < cpu+topology; sibling++ {
+					cpuTopologyDir := filepath.Join(testCPUDir, fmt.Sprintf("cpu%d", sibling), "topology")
+					err := os.MkdirAll(cpuTopologyDir, os.ModePerm)
+					Expect(err).ToNot(HaveOccurred())
+					siblingsFile := filepath.Join(cpuTopologyDir, "thread_siblings_list")
+					err = os.WriteFile(siblingsFile, []byte(siblingsSet.String()), 0o644)
+					Expect(err).ToNot(HaveOccurred())
+				}
+			}
+		}
+
+		createInvalidSysCPUThreadSiblingsDir := func(testCPUDir string, numCPUs int) {
+			err := os.MkdirAll(testCPUDir, os.ModePerm)
+			Expect(err).ToNot(HaveOccurred())
+
+			// Create CPU directories and topology files for CPUs.
+			for cpu := range numCPUs {
+				cpuTopologyDir := filepath.Join(testCPUDir, fmt.Sprintf("cpu%d", cpu), "topology")
+				err := os.MkdirAll(cpuTopologyDir, os.ModePerm)
+				Expect(err).ToNot(HaveOccurred())
+				siblingsFile := filepath.Join(cpuTopologyDir, "thread_siblings_list")
+				err = os.WriteFile(siblingsFile, []byte("invalid"), 0o644)
+				Expect(err).ToNot(HaveOccurred())
+			}
+		}
+
+		createContainerSandbox := func(cpus string, annotations map[string]string) (*oci.Container, *sandbox.Sandbox) {
+			c, err := oci.NewContainer("containerID", "", "", "",
+				make(map[string]string), make(map[string]string),
+				make(map[string]string), "pauseImage", nil, nil, "",
+				&types.ContainerMetadata{}, "sandboxID", false, false,
+				false, "", "", time.Now(), "")
+			Expect(err).ToNot(HaveOccurred())
+			c.SetSpec(
+				&specs.Spec{
+					Linux: &specs.Linux{
+						Resources: &specs.LinuxResources{
+							CPU: &specs.LinuxCPU{
+								Cpus: cpus,
+							},
+						},
+					},
+				},
+			)
+			sbox := baseSandboxBuilder()
+			err = sbox.SetCRISandbox(sbox.ID(), make(map[string]string), annotations, &types.PodSandboxMetadata{})
+			Expect(err).ToNot(HaveOccurred())
+			sb, err := sbox.GetSandbox()
+			Expect(err).ToNot(HaveOccurred())
+
+			return c, sb
+		}
+
+		verifySetIRQLoadBalancing := func(c *oci.Container, sb *sandbox.Sandbox, enabled bool,
+			expectedSmp, expectedBan, expectedHousekeepingCPUs string, expectFailure bool,
+		) {
+			h := &HighPerformanceHooks{
+				irqBalanceConfigFile: irqBalanceConfigFile,
+				irqSMPAffinityFile:   irqSmpAffinityFile,
+				sysCPUDir:            sysCPUDir,
+			}
+
+			// For container start (enabled == false), we must calculate the housekeeping CPUs,
+			// otherwise housekeepingSiblings is the empty set.
+			housekeepingSiblings := cpuset.CPUSet{}
+			if !enabled {
+				spec := c.Spec()
+				housekeepingSiblings, err = h.getHousekeepingCPUs(&spec, sb.Annotations())
+				if expectFailure {
+					Expect(err).To(HaveOccurred())
+
+					return
+				} else {
+					Expect(err).ToNot(HaveOccurred())
+				}
+			}
+
+			err = h.setIRQLoadBalancing(context.TODO(), c, housekeepingSiblings, enabled)
+			Expect(err).ToNot(HaveOccurred())
+
+			content, err := os.ReadFile(irqSmpAffinityFile)
+			Expect(err).ToNot(HaveOccurred())
+
+			Expect(strings.Trim(string(content), "\n")).To(Equal(expectedSmp))
+
+			bannedCPUs, err := retrieveIrqBannedCPUMasks(irqBalanceConfigFile)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(bannedCPUs).To(Equal(expectedBan))
+
+			// Also test that injection via injectHousekeepingEnv works correctly.
+			specgen := generate.NewFromSpec(&specs.Spec{Process: &specs.Process{}})
+			if !housekeepingSiblings.IsEmpty() {
+				err := injectHousekeepingEnv(&specgen, housekeepingSiblings)
+				Expect(err).ToNot(HaveOccurred())
+			}
+			var expectedHousekeepingAnnotation []string
+			if expectedHousekeepingCPUs != "" {
+				expectedHousekeepingAnnotation = append(
+					expectedHousekeepingAnnotation,
+					fmt.Sprintf("%s=%s", HousekeepingCPUsEnvVar, expectedHousekeepingCPUs),
+				)
+			}
+			Expect(specgen.Config.Process.Env).To(Equal(expectedHousekeepingAnnotation))
+		}
+
+		JustBeforeEach(func() {
+			err = os.WriteFile(irqBalanceConfigFile, []byte(""), 0o644)
+			Expect(err).ToNot(HaveOccurred())
+			err = updateIrqBalanceConfigFile(irqBalanceConfigFile, bannedCPUFlags)
+			Expect(err).ToNot(HaveOccurred())
+			bannedCPUs, err := retrieveIrqBannedCPUMasks(irqBalanceConfigFile)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(bannedCPUs).To(Equal(bannedCPUFlags))
+			err = os.WriteFile(irqSmpAffinityFile, []byte(flags), 0o644)
+			Expect(err).ToNot(HaveOccurred())
+		})
+
+		// This should be covered in other tests already, but test this here for completeness and safe measure.
+		Context("with enabled equals to true", func() {
+			BeforeEach(func() {
+				flags = "00000000,ffffff0f"
+				bannedCPUFlags = "ffffffff,000000f0"
+			})
+
+			It("should set the irq bit mask with housekeeping CPUs annotation present", func() {
+				c, sb := createContainerSandbox("4,5,6,7", map[string]string{
+					crioannotations.IRQLoadBalancingAnnotation: annotationHousekeeping,
+				})
+				verifySetIRQLoadBalancing(c, sb, true, "00000000,ffffffff", "ffffffff,00000000", "", false)
+			})
+		})
+
+		Context("with enabled equals to false", func() {
+			BeforeEach(func() {
+				flags = "00000000,ffffffff"
+				bannedCPUFlags = "ffffffff,00000000"
+			})
+
+			It("should set the irq bit mask without housekeeping CPUs", func() {
+				c, sb := createContainerSandbox("4,5,6,7", map[string]string{})
+				verifySetIRQLoadBalancing(c, sb, false, "00000000,ffffff0f", "ffffffff,000000f0", "", false)
+			})
+
+			It("should set the irq bit mask with housekeeping CPUs when no thread siblings files are present", func() {
+				c, sb := createContainerSandbox("4,5,6,7", map[string]string{
+					crioannotations.IRQLoadBalancingAnnotation: annotationHousekeeping,
+				})
+				verifySetIRQLoadBalancing(c, sb, false, "00000000,ffffff1f", "ffffffff,000000e0", "4", false)
+			})
+
+			It("should set the irq bit mask with housekeeping CPUs and no siblings", func() {
+				createSysCPUThreadSiblingsDir(sysCPUDir, 64, 1)
+				c, sb := createContainerSandbox("4,5,6,7", map[string]string{
+					crioannotations.IRQLoadBalancingAnnotation: annotationHousekeeping,
+				})
+				verifySetIRQLoadBalancing(c, sb, false, "00000000,ffffff1f", "ffffffff,000000e0", "4", false)
+			})
+
+			It("should set the irq bit mask with housekeeping CPUs and siblings (topology 2)", func() {
+				createSysCPUThreadSiblingsDir(sysCPUDir, 64, 2)
+				c, sb := createContainerSandbox("4,5,6,7", map[string]string{
+					crioannotations.IRQLoadBalancingAnnotation: annotationHousekeeping,
+				})
+				verifySetIRQLoadBalancing(c, sb, false, "00000000,ffffff3f", "ffffffff,000000c0", "4-5", false)
+			})
+
+			It("should set the irq bit mask with housekeeping CPUs and siblings (topology 4)", func() {
+				createSysCPUThreadSiblingsDir(sysCPUDir, 64, 4)
+				c, sb := createContainerSandbox("4-11", map[string]string{
+					crioannotations.IRQLoadBalancingAnnotation: annotationHousekeeping,
+				})
+				verifySetIRQLoadBalancing(c, sb, false, "00000000,fffff0ff", "ffffffff,00000f00", "4-7", false)
+			})
+
+			It("should fail with invalid siblings files", func() {
+				createInvalidSysCPUThreadSiblingsDir(sysCPUDir, 64)
+				c, sb := createContainerSandbox("4-11", map[string]string{
+					crioannotations.IRQLoadBalancingAnnotation: annotationHousekeeping,
+				})
+				verifySetIRQLoadBalancing(c, sb, false, "00000000,fffff0ff", "ffffffff,00000f00", "4-7", true)
+			})
+		})
+	})
+
 	Describe("setCPUPMQOSResumeLatency", func() {
 		var pmQosResumeLatencyUs, pmQosResumeLatencyUsOriginal string
 		cpuDir := filepath.Join(fixturesDir, "cpu")
@@ -551,9 +847,12 @@ var _ = Describe("high_performance_hooks", func() {
 	})
 
 	Describe("restoreIrqBalanceConfig", func() {
+		var mockSvcMgr *mockServiceManager
+
 		irqSmpAffinityFile := filepath.Join(fixturesDir, "irq_smp_affinity")
 		irqBalanceConfigFile := filepath.Join(fixturesDir, "irqbalance")
 		irqBannedCPUConfigFile := filepath.Join(fixturesDir, "orig_irq_banned_cpus")
+
 		verifyRestoreIrqBalanceConfig := func(expectedOrigBannedCPUs, expectedBannedCPUs string) {
 			err = RestoreIrqBalanceConfig(context.TODO(), irqBalanceConfigFile, irqBannedCPUConfigFile, irqSmpAffinityFile)
 			ExpectWithOffset(1, err).ToNot(HaveOccurred())
@@ -579,6 +878,21 @@ var _ = Describe("high_performance_hooks", func() {
 			bannedCPUs, err := retrieveIrqBannedCPUMasks(irqBalanceConfigFile)
 			Expect(err).ToNot(HaveOccurred())
 			Expect(bannedCPUs).To(Equal("0000ffff,ffffcfcc"))
+
+			mockSvcMgr = &mockServiceManager{
+				isServiceEnabled: map[string]bool{
+					"irqbalance": true,
+				},
+				restartService: map[string]error{
+					"irqbalance": nil,
+				},
+				history: []string{},
+			}
+			serviceManager = mockSvcMgr
+		})
+
+		JustAfterEach(func() {
+			serviceManager = &defaultServiceManager{}
 		})
 
 		Context("when banned cpu config file doesn't exist", func() {
@@ -589,6 +903,8 @@ var _ = Describe("high_performance_hooks", func() {
 
 			It("should set banned cpu config file from irq balance config", func() {
 				verifyRestoreIrqBalanceConfig("0000ffff,ffffcfcc", "0000ffff,ffffcfcc")
+				Expect(mockSvcMgr.history).NotTo(ContainElement("systemctl is-enabled irqbalance"))
+				Expect(mockSvcMgr.history).NotTo(ContainElement("systemctl restart irqbalance"))
 			})
 		})
 
@@ -602,10 +918,189 @@ var _ = Describe("high_performance_hooks", func() {
 
 			It("should restore irq balance config with content from banned cpu config file", func() {
 				verifyRestoreIrqBalanceConfig("00000000,00000000", "00000000,00000000")
+				Expect(mockSvcMgr.history).To(ContainElement("systemctl is-enabled irqbalance"))
+				Expect(mockSvcMgr.history).To(ContainElement("systemctl restart irqbalance"))
 			})
 		})
 	})
 
+	Describe("handleIRQBalanceRestart", func() {
+		irqBalanceConfigFile := filepath.Join(fixturesDir, "irqbalance")
+
+		h := &HighPerformanceHooks{
+			irqBalanceConfigFile: irqBalanceConfigFile,
+		}
+
+		type parameters struct {
+			isServiceEnabled         bool
+			irqBalanceFileExists     bool
+			restartServiceSucceeds   bool
+			pathLookupError          bool
+			calculatedIRQBalanceMask string
+		}
+
+		DescribeTable("handleIRQBalanceRestart scenarios",
+			func(p parameters, serviceMgrHistory, cmdRunnerHistory []string) {
+				defer func() {
+					// Reset global mocks.
+					serviceManager = &defaultServiceManager{}
+					commandRunner = &defaultCommandRunner{}
+				}()
+
+				// Setup mocks according to parameters and irqbalance config file.
+				mockSvcMgr := &mockServiceManager{
+					isServiceEnabled: map[string]bool{
+						"irqbalance": p.isServiceEnabled,
+					},
+					history: []string{},
+				}
+				mockCmdRunner := &mockCommandRunner{
+					history: []string{},
+				}
+
+				if p.restartServiceSucceeds {
+					mockSvcMgr.restartService = map[string]error{
+						"irqbalance": nil,
+					}
+				} else {
+					mockSvcMgr.restartService = map[string]error{
+						"irqbalance": errors.New("restart failed"),
+					}
+				}
+
+				if p.pathLookupError {
+					mockCmdRunner.lookPath = map[string]struct {
+						path string
+						err  error
+					}{
+						"irqbalance": {path: "", err: errors.New("not found")},
+					}
+				} else {
+					mockCmdRunner.lookPath = map[string]struct {
+						path string
+						err  error
+					}{
+						"irqbalance": {path: "/usr/bin/irqbalance", err: nil},
+					}
+				}
+				if p.irqBalanceFileExists {
+					err = os.WriteFile(irqBalanceConfigFile, []byte(""), 0o644)
+					Expect(err).ToNot(HaveOccurred())
+					err = updateIrqBalanceConfigFile(irqBalanceConfigFile, p.calculatedIRQBalanceMask)
+					Expect(err).ToNot(HaveOccurred())
+				}
+				serviceManager = mockSvcMgr
+				commandRunner = mockCmdRunner
+
+				// Execute application logic.
+				if !h.handleIRQBalanceRestart(context.TODO(), "container-name") {
+					h.handleIRQBalanceOneShot(context.TODO(), "container-name", p.calculatedIRQBalanceMask)
+				}
+
+				// Verify behavior based on scenario.
+				Expect(mockSvcMgr.history).To(Equal(serviceMgrHistory))
+				Expect(mockCmdRunner.history).To(Equal(cmdRunnerHistory))
+			},
+			Entry("irqbalance is enabled and succeeds",
+				parameters{isServiceEnabled: true, irqBalanceFileExists: true, restartServiceSucceeds: true, pathLookupError: false, calculatedIRQBalanceMask: "ffff,ffff"},
+				[]string{
+					"systemctl is-enabled irqbalance",
+					"systemctl restart irqbalance",
+				},
+				[]string{}),
+			Entry("irqbalance is enabled but irqbalance file does not exist",
+				parameters{isServiceEnabled: true, irqBalanceFileExists: false, restartServiceSucceeds: false, pathLookupError: false, calculatedIRQBalanceMask: "ffff,ffff"},
+				[]string{
+					"systemctl is-enabled irqbalance",
+				},
+				[]string{
+					"which irqbalance",
+					"IRQBALANCE_BANNED_CPUS=ffff,ffff /usr/bin/irqbalance --oneshot",
+				}),
+			Entry("irqbalance is enabled and fails but oneshot works",
+				parameters{isServiceEnabled: true, irqBalanceFileExists: true, restartServiceSucceeds: false, pathLookupError: false, calculatedIRQBalanceMask: "ffff,ffff"},
+				[]string{
+					"systemctl is-enabled irqbalance",
+					"systemctl restart irqbalance",
+				},
+				[]string{
+					"which irqbalance",
+					"IRQBALANCE_BANNED_CPUS=ffff,ffff /usr/bin/irqbalance --oneshot",
+				}),
+			Entry("irqbalance is disabled but irqBalance file exists",
+				parameters{isServiceEnabled: false, irqBalanceFileExists: true, restartServiceSucceeds: false, pathLookupError: false, calculatedIRQBalanceMask: "ffff,ffff"},
+				[]string{
+					"systemctl is-enabled irqbalance",
+				},
+				[]string{
+					"which irqbalance",
+					"IRQBALANCE_BANNED_CPUS=ffff,ffff /usr/bin/irqbalance --oneshot",
+				}),
+			Entry("irqbalance is disabled, oneshot lookup fails",
+				parameters{isServiceEnabled: false, irqBalanceFileExists: true, restartServiceSucceeds: false, pathLookupError: true, calculatedIRQBalanceMask: "ffff,ffff"},
+				[]string{
+					"systemctl is-enabled irqbalance",
+				},
+				[]string{
+					"which irqbalance",
+				}),
+		)
+	})
+
+	Describe("updateNewIRQSMPAffinityMask rollback", func() {
+		irqBalanceConfigFile := filepath.Join(fixturesDir, "irqbalance")
+		irqSMPAffinityFile := filepath.Join(fixturesDir, "irqsmpaffinity")
+
+		h := &HighPerformanceHooks{
+			irqSMPAffinityFile:   irqSMPAffinityFile,
+			irqBalanceConfigFile: irqBalanceConfigFile,
+		}
+
+		type parameters struct {
+			irqBalanceFileRO           bool
+			originalIRQSMPAffinityMask string
+			expectedIRQSMPAffinityMask string
+		}
+
+		DescribeTable("test rollback",
+			func(p parameters) {
+				err := os.WriteFile(irqSMPAffinityFile, []byte(p.originalIRQSMPAffinityMask), 0o644)
+				Expect(err).ToNot(HaveOccurred())
+
+				if p.irqBalanceFileRO {
+					err = os.Symlink("/proc/version", irqBalanceConfigFile)
+					Expect(err).ToNot(HaveOccurred())
+				} else {
+					err = os.WriteFile(irqBalanceConfigFile, []byte(""), 0o644)
+					Expect(err).ToNot(HaveOccurred())
+				}
+
+				_, err = h.updateNewIRQSMPAffinityMask(context.TODO(), "cID", "CName", cpuSetOrDie("2-3"), false)
+				if p.irqBalanceFileRO {
+					Expect(err).To(HaveOccurred())
+				} else {
+					Expect(err).NotTo(HaveOccurred())
+				}
+
+				writtenMask, err := os.ReadFile(irqSMPAffinityFile)
+				Expect(err).NotTo(HaveOccurred())
+				Expect(string(writtenMask)).To(Equal(p.expectedIRQSMPAffinityMask))
+			},
+			Entry("writing IRQ balance file fails",
+				parameters{
+					irqBalanceFileRO:           true,
+					originalIRQSMPAffinityMask: "ffffffff",
+					expectedIRQSMPAffinityMask: "ffffffff",
+				}),
+			Entry("writing IRQ balance file succeeds",
+				parameters{
+					irqBalanceFileRO:           false,
+					originalIRQSMPAffinityMask: "ffffffff",
+					expectedIRQSMPAffinityMask: "fffffff3",
+				}),
+		)
+	})
+
 	Describe("convertAnnotationToLatency", func() {
 		verifyConvertAnnotationToLatency := func(annotation string, expected string, expect_error bool) {
 			latency, err := convertAnnotationToLatency(annotation)
@@ -706,38 +1201,10 @@ var _ = Describe("high_performance_hooks", func() {
 		})
 	})
 	Describe("PreCreate Hook", func() {
-		baseSandboxBuilder := func() sandbox.Builder {
-			sbox := sandbox.NewBuilder()
-			createdAt := time.Now()
-			sbox.SetCreatedAt(createdAt)
-			sbox.SetID("sandboxID")
-			sbox.SetName("sandboxName")
-			sbox.SetLogDir("test")
-			sbox.SetShmPath("test")
-			sbox.SetNamespace("")
-			sbox.SetKubeName("")
-			sbox.SetMountLabel("test")
-			sbox.SetProcessLabel("test")
-			sbox.SetCgroupParent("")
-			sbox.SetRuntimeHandler("")
-			sbox.SetResolvPath("")
-			sbox.SetHostname("")
-			sbox.SetPortMappings([]*hostport.PortMapping{})
-			sbox.SetHostNetwork(false)
-			sbox.SetUsernsMode("")
-			sbox.SetPodLinuxOverhead(nil)
-			sbox.SetPodLinuxResources(nil)
-			sbox.SetPrivileged(false)
-			sbox.SetHostNetwork(false)
-			sbox.SetCreatedAt(createdAt)
-
-			return sbox
-		}
-
 		shares := uint64(2048)
 		baseGenerator := func() *generate.Generator {
-			return &generate.Generator{
-				Config: &specs.Spec{
+			g := generate.NewFromSpec(
+				&specs.Spec{
 					Process: &specs.Process{
 						Env: make([]string, 0),
 					},
@@ -749,7 +1216,9 @@ var _ = Describe("high_performance_hooks", func() {
 						},
 					},
 				},
-			}
+			)
+
+			return &g
 		}
 
 		buildContainer := func(g *generate.Generator) (*oci.Container, error) {
@@ -924,16 +1393,25 @@ var _ = Describe("high_performance_hooks", func() {
 		var cfg *config.Config
 		var hooksRetriever *HooksRetriever
 
+		formatIRQBalanceBannedCPUs := func(v string) string {
+			return fmt.Sprintf("%s=%q", irqBalanceBannedCpus, v)
+		}
+
 		irqSmpAffinityFile := filepath.Join(fixturesDir, "irq_smp_affinity")
 		irqBalanceConfigFile := filepath.Join(fixturesDir, "irqbalance")
-		flags = "0000,0000ffff"
+		flags := "0000,0000ffff"
+		bannedCPUFlags = "ffffffff,ffff0000"
 
 		ctx := context.Background()
 
-		verifySetIRQLoadBalancing := func(expected string) {
+		verifySetIRQLoadBalancing := func(expectedIrqSmp, expectedIrqBalance string) {
 			content, err := os.ReadFile(irqSmpAffinityFile)
 			Expect(err).ToNot(HaveOccurred())
-			Expect(strings.Trim(string(content), "\n")).To(Equal(expected))
+			Expect(strings.Trim(string(content), "\n")).To(Equal(expectedIrqSmp))
+
+			content, err = os.ReadFile(irqBalanceConfigFile)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(strings.Trim(string(content), "\n")).To(Equal(formatIRQBalanceBannedCPUs(expectedIrqBalance)))
 		}
 
 		createContainer := func(cpus string) (*oci.Container, error) {
@@ -971,6 +1449,8 @@ var _ = Describe("high_performance_hooks", func() {
 			// create tests affinity file
 			err = os.WriteFile(irqSmpAffinityFile, []byte(flags), 0o644)
 			Expect(err).ToNot(HaveOccurred())
+			err = os.WriteFile(irqBalanceConfigFile, []byte(formatIRQBalanceBannedCPUs(bannedCPUFlags)), 0o644)
+			Expect(err).ToNot(HaveOccurred())
 
 			sbox := sandbox.NewBuilder()
 			createdAt := time.Now()
@@ -1028,6 +1508,7 @@ var _ = Describe("high_performance_hooks", func() {
 				Expect(hooks).NotTo(BeNil())
 				if hph, ok := hooks.(*HighPerformanceHooks); ok {
 					hph.irqSMPAffinityFile = irqSmpAffinityFile
+					hph.irqBalanceConfigFile = irqBalanceConfigFile
 				}
 				var wg sync.WaitGroup
 				for cpu := range 16 {
@@ -1041,7 +1522,7 @@ var _ = Describe("high_performance_hooks", func() {
 					}()
 				}
 				wg.Wait()
-				verifySetIRQLoadBalancing("00000000,00000000")
+				verifySetIRQLoadBalancing("00000000,00000000", "ffffffff,ffffffff")
 			})
 		})
 
@@ -1068,6 +1549,7 @@ var _ = Describe("high_performance_hooks", func() {
 				hph, ok := hooks.(*HighPerformanceHooks)
 				Expect(ok).To(BeTrue())
 				hph.irqSMPAffinityFile = irqSmpAffinityFile
+				hph.irqBalanceConfigFile = irqBalanceConfigFile
 
 				var wg sync.WaitGroup
 				for cpu := range 16 {
@@ -1081,7 +1563,7 @@ var _ = Describe("high_performance_hooks", func() {
 					}()
 				}
 				wg.Wait()
-				verifySetIRQLoadBalancing(flags)
+				verifySetIRQLoadBalancing(flags, bannedCPUFlags)
 			})
 		})
 
@@ -1109,6 +1591,7 @@ var _ = Describe("high_performance_hooks", func() {
 				Expect(hooks).NotTo(BeNil())
 				if hph, ok := hooks.(*HighPerformanceHooks); ok {
 					hph.irqSMPAffinityFile = irqSmpAffinityFile
+					hph.irqBalanceConfigFile = irqBalanceConfigFile
 				}
 				var wg sync.WaitGroup
 				for cpu := range 16 {
@@ -1122,7 +1605,7 @@ var _ = Describe("high_performance_hooks", func() {
 					}()
 				}
 				wg.Wait()
-				verifySetIRQLoadBalancing("00000000,00000000")
+				verifySetIRQLoadBalancing("00000000,00000000", "ffffffff,ffffffff")
 			})
 		})
 
@@ -1173,6 +1656,7 @@ var _ = Describe("high_performance_hooks", func() {
 				Expect(hooks).NotTo(BeNil())
 				if hph, ok := hooks.(*HighPerformanceHooks); ok {
 					hph.irqSMPAffinityFile = irqSmpAffinityFile
+					hph.irqBalanceConfigFile = irqBalanceConfigFile
 				}
 				var wg sync.WaitGroup
 				for cpu := range 16 {
@@ -1186,7 +1670,7 @@ var _ = Describe("high_performance_hooks", func() {
 					}()
 				}
 				wg.Wait()
-				verifySetIRQLoadBalancing("00000000,00000000")
+				verifySetIRQLoadBalancing("00000000,00000000", "ffffffff,ffffffff")
 			})
 		})
 
@@ -1234,7 +1718,7 @@ var _ = Describe("high_performance_hooks", func() {
 					}()
 				}
 				wg.Wait()
-				verifySetIRQLoadBalancing(flags)
+				verifySetIRQLoadBalancing(flags, bannedCPUFlags)
 			})
 		})
 	})
