@@ -14,6 +14,7 @@ import (
 	"sync"
 	"time"
 
+	"github.com/containers/common/libimage"
 	"github.com/containers/image/v5/copy"
 	"github.com/containers/image/v5/docker"
 	"github.com/containers/image/v5/docker/reference"
@@ -36,6 +37,7 @@ import (
 	crierrors "k8s.io/cri-api/pkg/errors"
 
 	"github.com/cri-o/cri-o/internal/log"
+	"github.com/cri-o/cri-o/internal/ociartifact"
 	"github.com/cri-o/cri-o/internal/storage/references"
 	"github.com/cri-o/cri-o/pkg/config"
 )
@@ -60,7 +62,6 @@ type ImageResult struct {
 	RepoDigests         []string
 	Size                *uint64
 	Digest              digest.Digest
-	ConfigDigest        digest.Digest
 	User                string
 	PreviousName        string
 	Labels              map[string]string
@@ -81,11 +82,10 @@ type indexInfo struct {
 // Every field in imageCacheItem are fixed properties of an "image", which in this
 // context is the image.ID stored in c/storage, and thus don't need to be recomputed.
 type imageCacheItem struct {
-	config       *specs.Image
-	size         *uint64
-	configDigest digest.Digest
-	info         *types.ImageInspectInfo
-	annotations  map[string]string
+	config      *specs.Image
+	size        *uint64
+	info        *types.ImageInspectInfo
+	annotations map[string]string
 }
 
 type imageCache map[string]imageCacheItem
@@ -139,9 +139,6 @@ type ImageServer interface {
 	// ImageStatusByName returns status of an image tagged with name.
 	ImageStatusByName(systemContext *types.SystemContext, name RegistryImageReference) (*ImageResult, error)
 
-	// PrepareImage returns an Image where the config digest can be grabbed
-	// for further analysis. Call Close() on the resulting image.
-	PrepareImage(systemContext *types.SystemContext, imageName RegistryImageReference) (types.ImageCloser, error)
 	// PullImage imports an image from the specified location.
 	//
 	// Arguments:
@@ -150,10 +147,11 @@ type ImageServer interface {
 	// - options: Pointer to ImageCopyOptions, which contains various options for the image copy process
 	//
 	// Returns:
-	// - types.ImageReference: The reference to the pulled image
-	// - reference.Canonical: A canonical reference to the image, typically including the digest
+	// - A name@digest value referring to exactly the pulled image (the reference might become dangling if the image
+	//   is removed, but it will not ever match a different image). The value is suitable for PullImageResponse.ImageRef
+	//   and for ContainerConfig.Image.Image.
 	// - error: An error object if pulling the image fails, otherwise nil
-	PullImage(ctx context.Context, imageName RegistryImageReference, options *ImageCopyOptions) (types.ImageReference, reference.Canonical, error)
+	PullImage(ctx context.Context, imageName RegistryImageReference, options *ImageCopyOptions) (RegistryImageReference, error)
 
 	// DeleteImage deletes a storage image (impacting all its tags)
 	DeleteImage(systemContext *types.SystemContext, id StorageImageID) error
@@ -181,10 +179,10 @@ type ImageServer interface {
 	//
 	// Arguments:
 	// - ctx: The context for controlling the function's execution
-	// - systemContext: server's system context for the given namespace
-	// - imageName: A RegistryImageReference that identifies the container image
+	// - systemContext: server's system context for the given namespace, notably it might have a customized SignaturePolicyPath.
+	// - userSpecifiedImage: a RegistryImageReference that expresses users’ _intended_ image.
 	// - imageID: A StorageImageID of the image
-	IsRunningImageAllowed(ctx context.Context, systemContext *types.SystemContext, imageName RegistryImageReference, imageID StorageImageID) error
+	IsRunningImageAllowed(ctx context.Context, systemContext *types.SystemContext, userSpecifiedImage RegistryImageReference, imageID StorageImageID) error
 }
 
 func parseImageNames(image *storage.Image) (someName *RegistryImageReference, tags []reference.NamedTagged, digests []reference.Canonical, err error) {
@@ -193,9 +191,11 @@ func parseImageNames(image *storage.Image) (someName *RegistryImageReference, ta
 		if err != nil {
 			return nil, nil, nil, fmt.Errorf("invalid name %q in image %q: %w", nameString, image.ID, err)
 		}
+
 		if reference.IsNameOnly(name) {
 			return nil, nil, nil, fmt.Errorf("invalid name %q in image %q, it has neither a tag nor a digest", nameString, image.ID)
 		}
+
 		switch name := name.(type) {
 		case reference.Canonical:
 			digests = append(digests, name)
@@ -205,14 +205,17 @@ func parseImageNames(image *storage.Image) (someName *RegistryImageReference, ta
 			return nil, nil, nil, fmt.Errorf("internal error, invalid name %q in image %q is !IsNameOnly but neither Canonical nor NamedTagged", nameString, image.ID)
 		}
 	}
+
 	if len(digests) > 0 {
 		best := references.RegistryImageReferenceFromRaw(digests[0])
 		someName = &best
 	}
+
 	if len(tags) > 0 {
 		best := references.RegistryImageReferenceFromRaw(tags[0])
 		someName = &best
 	}
+
 	return someName, tags, digests, nil
 }
 
@@ -224,9 +227,12 @@ func (svc *imageService) makeRepoDigests(knownRepoDigests []reference.Canonical,
 		if err != nil || imgDigest == "" {
 			return "", knownRepoDigests
 		}
+
 		imageDigest = imgDigest
 	}
+
 	imageDigests := []digest.Digest{imageDigest}
+
 	for _, anotherImageDigest := range img.Digests {
 		if anotherImageDigest != imageDigest {
 			imageDigests = append(imageDigests, anotherImageDigest)
@@ -236,6 +242,7 @@ func (svc *imageService) makeRepoDigests(knownRepoDigests []reference.Canonical,
 	// that we already know.
 	digestMap := make(map[string]struct{})
 	repoDigests = knownRepoDigests
+
 	for _, repoDigest := range knownRepoDigests {
 		digestMap[repoDigest.String()] = struct{}{}
 	}
@@ -244,6 +251,7 @@ func (svc *imageService) makeRepoDigests(knownRepoDigests []reference.Canonical,
 	for _, tagged := range tags {
 		repos = append(repos, reference.TrimNamed(tagged))
 	}
+
 	for _, digested := range knownRepoDigests {
 		repos = append(repos, reference.TrimNamed(digested))
 	}
@@ -259,6 +267,7 @@ func (svc *imageService) makeRepoDigests(knownRepoDigests []reference.Canonical,
 			}
 		}
 	}
+
 	return imageDigest, repoDigests
 }
 
@@ -267,12 +276,14 @@ func (svc *imageService) buildImageCacheItem(systemContext *types.SystemContext,
 	if err != nil {
 		return imageCacheItem{}, err
 	}
+
 	defer imageFull.Close()
-	configDigest := imageFull.ConfigInfo().Digest
+
 	imageConfig, err := imageFull.OCIConfig(svc.ctx)
 	if err != nil {
 		return imageCacheItem{}, err
 	}
+
 	size := imageSize(imageFull)
 
 	info, err := imageFull.Inspect(svc.ctx)
@@ -290,6 +301,7 @@ func (svc *imageService) buildImageCacheItem(systemContext *types.SystemContext,
 	if err != nil {
 		return imageCacheItem{}, err
 	}
+
 	var ociManifest specs.Manifest
 	if manifestType == specs.MediaTypeImageManifest {
 		if err := json.Unmarshal(topManifestBlob, &ociManifest); err != nil {
@@ -298,11 +310,10 @@ func (svc *imageService) buildImageCacheItem(systemContext *types.SystemContext,
 	}
 
 	return imageCacheItem{
-		config:       imageConfig,
-		size:         size,
-		configDigest: configDigest,
-		info:         info,
-		annotations:  ociManifest.Annotations,
+		config:      imageConfig,
+		size:        size,
+		info:        info,
+		annotations: ociManifest.Annotations,
 	}, nil
 }
 
@@ -311,20 +322,25 @@ func (svc *imageService) buildImageResult(image *storage.Image, cacheItem imageC
 	if err != nil {
 		return ImageResult{}, err
 	}
+
 	imageDigest, repoDigests := svc.makeRepoDigests(digests, tags, image)
 
 	repoTagStrings := make([]string, 0, len(tags))
 	for _, t := range tags {
 		repoTagStrings = append(repoTagStrings, t.String())
 	}
+
 	sort.Strings(repoTagStrings)
+
 	repoDigestStrings := make([]string, 0, len(repoDigests))
 	for _, d := range repoDigests {
 		repoDigestStrings = append(repoDigestStrings, d.String())
 	}
+
 	sort.Strings(repoDigestStrings)
 
 	previousName := ""
+
 	if len(image.NamesHistory) > 0 {
 		// Remove the tag because we can only keep the name as indicator
 		split := strings.SplitN(image.NamesHistory[0], ":", 2)
@@ -334,9 +350,11 @@ func (svc *imageService) buildImageResult(image *storage.Image, cacheItem imageC
 	}
 
 	imagePinned := false
+
 	for _, image := range image.Names {
 		if FilterPinnedImage(image, svc.regexForPinnedImages) {
 			imagePinned = true
+
 			break
 		}
 	}
@@ -353,11 +371,14 @@ func (svc *imageService) buildImageResult(image *storage.Image, cacheItem imageC
 			logrus.Warnf("Unable to get mount info for path %s: %v", mountPoint, err)
 			mountPoint = ""
 		}
+
 		if len(infos) == 0 {
 			logrus.Warnf("Unable to find mount path %s for image %s, assuming image is no longer mounted", mountPoint, image.ID)
+
 			if _, err := svc.store.UnmountImage(image.ID, true); err != nil {
 				logrus.Warnf("Unable to unmount image %s: %v", image.ID, err)
 			}
+
 			mountPoint = ""
 		}
 	}
@@ -369,7 +390,6 @@ func (svc *imageService) buildImageResult(image *storage.Image, cacheItem imageC
 		RepoDigests:         repoDigestStrings,
 		Size:                cacheItem.size,
 		Digest:              imageDigest,
-		ConfigDigest:        cacheItem.configDigest,
 		User:                cacheItem.config.Config.User,
 		PreviousName:        previousName,
 		Labels:              cacheItem.info.Labels,
@@ -385,32 +405,40 @@ func (svc *imageService) ListImages(systemContext *types.SystemContext) ([]Image
 	if err != nil {
 		return nil, err
 	}
+
 	results := make([]ImageResult, 0, len(images))
 	newImageCache := make(imageCache, len(images))
+
 	for i := range images {
 		image := &images[i]
+
 		ref, err := istorage.Transport.NewStoreReference(svc.store, nil, image.ID)
 		if err != nil {
 			return nil, err
 		}
+
 		svc.imageCacheLock.Lock()
 		cacheItem, ok := svc.imageCache[image.ID]
 		svc.imageCacheLock.Unlock()
+
 		if !ok {
 			cacheItem, err = svc.buildImageCacheItem(systemContext, ref)
 			if err != nil {
 				if os.IsNotExist(err) && imageIsBeingPulled(image) { // skip reporting errors if the images haven't finished pulling
 					continue
 				}
+
 				return nil, err
 			}
 		}
 
 		newImageCache[image.ID] = cacheItem
+
 		res, err := svc.buildImageResult(image, cacheItem)
 		if err != nil {
 			return nil, err
 		}
+
 		results = append(results, res)
 	}
 	// replace image cache with cache we just built
@@ -418,6 +446,7 @@ func (svc *imageService) ListImages(systemContext *types.SystemContext) ([]Image
 	svc.imageCacheLock.Lock()
 	svc.imageCache = newImageCache
 	svc.imageCacheLock.Unlock()
+
 	return results, nil
 }
 
@@ -427,6 +456,7 @@ func imageIsBeingPulled(image *storage.Image) bool {
 			return true
 		}
 	}
+
 	return false
 }
 
@@ -435,6 +465,7 @@ func (svc *imageService) ImageStatusByName(systemContext *types.SystemContext, n
 	if err != nil {
 		return nil, err
 	}
+
 	return svc.imageStatus(systemContext, unstableRef)
 }
 
@@ -443,6 +474,7 @@ func (svc *imageService) ImageStatusByID(systemContext *types.SystemContext, id
 	if err != nil {
 		return nil, err
 	}
+
 	return svc.imageStatus(systemContext, ref)
 }
 
@@ -461,6 +493,7 @@ func (svc *imageService) imageStatus(systemContext *types.SystemContext, unstabl
 
 	if !ok {
 		var err error
+
 		cacheItem, err = svc.buildImageCacheItem(systemContext, resolvedRef) // Single-use-only, not actually cached
 		if err != nil {
 			return nil, err
@@ -471,10 +504,11 @@ func (svc *imageService) imageStatus(systemContext *types.SystemContext, unstabl
 	if err != nil {
 		return nil, err
 	}
+
 	return &result, nil
 }
 
-func (svc *imageService) IsRunningImageAllowed(ctx context.Context, systemContext *types.SystemContext, imageName RegistryImageReference, imageID StorageImageID) error {
+func (svc *imageService) IsRunningImageAllowed(ctx context.Context, systemContext *types.SystemContext, userSpecifiedImage RegistryImageReference, imageID StorageImageID) error {
 	policy, err := signature.DefaultPolicy(systemContext)
 	if err != nil {
 		return fmt.Errorf("get default policy: %w", err)
@@ -491,25 +525,35 @@ func (svc *imageService) IsRunningImageAllowed(ctx context.Context, systemContex
 		}
 	}()
 
-	if err := svc.checkSignature(ctx, systemContext, policyContext, imageName, imageID); err != nil {
-		return fmt.Errorf("checking signature of %q: %w", imageName, err)
+	if err := svc.checkSignature(ctx, systemContext, policyContext, userSpecifiedImage, imageID); err != nil {
+		return fmt.Errorf("checking signature of %q: %w", userSpecifiedImage, err)
 	}
 
-	log.Debugf(ctx, "Is allowed to run config image %s (policy path: %q)", imageName, systemContext.SignaturePolicyPath)
+	log.Debugf(ctx, "Is allowed to run config image %s (policy path: %q)", userSpecifiedImage, systemContext.SignaturePolicyPath)
 
 	return nil
 }
 
-func (svc *imageService) checkSignature(ctx context.Context, sys *types.SystemContext, policyContext *signature.PolicyContext, imageName RegistryImageReference, imageID StorageImageID) error {
-	userIdentityRef, err := docker.NewReference(imageName.Raw())
+func (svc *imageService) checkSignature(ctx context.Context, sys *types.SystemContext, policyContext *signature.PolicyContext, userSpecifiedImage RegistryImageReference, imageID StorageImageID) error {
+	userSpecifiedImageRef, err := docker.NewReference(userSpecifiedImage.Raw())
 	if err != nil {
-		return fmt.Errorf("creating docker:// reference for %q: %w", imageName.Raw().String(), err)
+		return fmt.Errorf("creating docker:// reference for %q: %w", userSpecifiedImage.Raw().String(), err)
 	}
 
+	// imageID is authoritative, but it may be a deduplicated image with several manifests,
+	// and only one of them might be signed with the signatures required by policy.
+	//
+	// Here we could, possibly:
+	// - if userSpecifiedImage is a repo@digest, resolve up that image, CHECK THAT IT MATCHES storageID, and use that
+	//   reference (to use certainly the right digest)
+	// - if userSpecifiedImage is a repo:tag, resolve up that image, CHECK THAT IT MATCHES storageID, and use that
+	//   reference (assuming some future c/storage that can map repo:tag to the right digest)
+	// Failing that (e.g. if a subsequent pull moved the tag, or if the image was untagged), try with the raw imageID.
 	storageRef, err := imageID.imageRef(svc)
 	if err != nil {
 		return fmt.Errorf("creating containers-storage: reference for %v: %w", storageRef, err)
 	}
+
 	log.Debugf(ctx, "Created storageRef = %q", transports.ImageName(storageRef))
 
 	storageSource, err := storageRef.NewImageSource(ctx, sys)
@@ -519,12 +563,14 @@ func (svc *imageService) checkSignature(ctx context.Context, sys *types.SystemCo
 	defer storageSource.Close()
 
 	unparsedToplevel := cimage.UnparsedInstance(storageSource, nil)
+
 	topManifest, topMIMEType, err := unparsedToplevel.Manifest(ctx)
 	if err != nil {
 		return fmt.Errorf("get top level manifest: %w", err)
 	}
 
 	unparsedInstance := unparsedToplevel
+
 	if manifest.MIMETypeIsMultiImage(topMIMEType) {
 		manifestList, err := manifest.ListFromBlob(topManifest, topMIMEType)
 		if err != nil {
@@ -539,16 +585,13 @@ func (svc *imageService) checkSignature(ctx context.Context, sys *types.SystemCo
 		unparsedInstance = cimage.UnparsedInstance(storageSource, &instanceDigest)
 	}
 
-	mixedUnparsedInstance := cimage.UnparsedInstanceWithReference(unparsedInstance, userIdentityRef)
+	mixedUnparsedInstance := cimage.UnparsedInstanceWithReference(unparsedInstance, userSpecifiedImageRef)
 
 	allowed, err := policyContext.IsRunningImageAllowed(ctx, mixedUnparsedInstance)
 	if err != nil {
-		var policyErr signature.PolicyRequirementError
-		if errors.As(err, &policyErr) {
-			return fmt.Errorf("%w: %w", crierrors.ErrSignatureValidationFailed, err)
-		}
-		return fmt.Errorf("verifying signatures: %w", err)
+		return fmt.Errorf("verifying signatures: %w", WrapSignatureCRIErrorIfNeeded(err))
 	}
+
 	if !allowed {
 		panic("Internal inconsistency: IsRunningImageAllowed returned !allowed and no error when checking image signature")
 	}
@@ -556,11 +599,27 @@ func (svc *imageService) checkSignature(ctx context.Context, sys *types.SystemCo
 	return nil
 }
 
+// WrapSignatureCRIErrorIfNeeded wraps the CRI ErrSignatureValidationFailed if
+// the provided err qualifies for that. If not, then it returns err.
+func WrapSignatureCRIErrorIfNeeded(err error) error {
+	var (
+		policyErr    signature.PolicyRequirementError
+		signatureErr signature.InvalidSignatureError
+	)
+	if errors.As(err, &policyErr) || errors.As(err, &signatureErr) {
+		return fmt.Errorf("%w: %w", crierrors.ErrSignatureValidationFailed, err)
+	}
+
+	return err
+}
+
 func imageSize(img types.Image) *uint64 {
 	if sum, err := img.Size(); err == nil {
 		usum := uint64(sum)
+
 		return &usum
 	}
+
 	return nil
 }
 
@@ -575,32 +634,6 @@ func (svc *imageLookupService) remoteImageReference(imageName RegistryImageRefer
 	return alltransports.ParseImageName(svc.DefaultTransport + imageName.StringForOutOfProcessConsumptionOnly())
 }
 
-// prepareReference creates an image reference from an image string and returns an updated types.SystemContext (never nil) for the image.
-func (svc *imageLookupService) prepareReference(inputSystemContext *types.SystemContext, imageName RegistryImageReference) (*types.SystemContext, types.ImageReference, error) {
-	srcRef, err := svc.remoteImageReference(imageName)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	sc := types.SystemContext{}
-	if inputSystemContext != nil {
-		sc = *inputSystemContext // A shallow copy
-	}
-	if secure := svc.isSecureIndex(imageName.Registry()); !secure {
-		sc.DockerInsecureSkipTLSVerify = types.OptionalBoolTrue
-	}
-	return &sc, srcRef, nil
-}
-
-func (svc *imageService) PrepareImage(inputSystemContext *types.SystemContext, imageName RegistryImageReference) (types.ImageCloser, error) {
-	systemContext, srcRef, err := svc.lookup.prepareReference(inputSystemContext, imageName)
-	if err != nil {
-		return nil, err
-	}
-
-	return srcRef.NewImage(svc.ctx, systemContext)
-}
-
 type pullImageArgs struct {
 	Lookup       *imageLookupService
 	ImageName    string // In the format of RegistryImageReference.StringForOutOfProcessConsumptionOnly()
@@ -612,9 +645,7 @@ type pullImageArgs struct {
 
 type pullImageOutputItem struct {
 	Progress *types.ProgressProperties `json:",omitempty"`
-	Result   string                    `json:",omitempty"` // If not "", in the format of transport.ImageName()
-	Name     string                    `json:",omitempty"`
-	Digest   digest.Digest             `json:",omitempty"`
+	Result   string                    `json:",omitempty"` // If not "", in the format of RegistryImageReference.StringForOutOfProcessConsumptionOnly(), and always contains a digest.
 }
 
 func pullImageChild() {
@@ -644,23 +675,26 @@ func pullImageChild() {
 
 	output := make(chan pullImageOutputItem)
 	outputWritten := make(chan struct{})
+
 	go formatPullImageOutputItemGoroutine(os.Stdout, output, outputWritten)
 
 	progress := make(chan types.ProgressProperties)
+
 	go func() {
 		for p := range progress {
 			output <- pullImageOutputItem{Progress: &p}
 		}
 	}()
+
 	args.Options.Progress = progress
 
-	destRef, canonicalRef, err := pullImageImplementation(context.Background(), args.Lookup, store, imageName, args.Options)
+	canonicalRef, err := pullImageImplementation(context.Background(), args.Lookup, store, imageName, args.Options)
 	if err != nil {
 		fmt.Fprintf(os.Stderr, "%v", err)
 		os.Exit(1)
 	}
 
-	output <- pullImageOutputItem{Result: transports.ImageName(destRef), Name: canonicalRef.Name(), Digest: canonicalRef.Digest()}
+	output <- pullImageOutputItem{Result: canonicalRef.StringForOutOfProcessConsumptionOnly()}
 
 	close(output)
 	<-outputWritten
@@ -677,6 +711,7 @@ func formatPullImageOutputItemGoroutine(dest io.Writer, items <-chan pullImageOu
 	for item := range items {
 		stream.WriteVal(item)
 		stream.WriteRaw("\n")
+
 		if err := stream.Flush(); err != nil {
 			fmt.Fprintf(os.Stderr, "%v", err)
 			//nolint:gocritic // “exitAfterDefer: os.Exit will exit, and `defer func(){...}(...)` will not run”
@@ -687,26 +722,28 @@ func formatPullImageOutputItemGoroutine(dest io.Writer, items <-chan pullImageOu
 	}
 }
 
-func (svc *imageService) pullImageParent(ctx context.Context, imageName RegistryImageReference, parentCgroup string, options *ImageCopyOptions) (types.ImageReference, reference.Canonical, error) {
+func (svc *imageService) pullImageParent(ctx context.Context, imageName RegistryImageReference, parentCgroup string, options *ImageCopyOptions) (RegistryImageReference, error) {
 	progress := options.Progress
 	// the first argument imageName is not used by the re-execed command but it is useful for debugging as it
 	// shows in the ps output.
 	cmd := reexec.CommandContext(ctx, "crio-pull-image", imageName.StringForOutOfProcessConsumptionOnly())
+
 	stdout, err := cmd.StdoutPipe()
 	if err != nil {
-		return nil, nil, fmt.Errorf("error getting stdout pipe for image copy process: %w", err)
+		return RegistryImageReference{}, fmt.Errorf("error getting stdout pipe for image copy process: %w", err)
 	}
+
 	defer stdout.Close()
 
 	stderr, err := cmd.StderrPipe()
 	if err != nil {
-		return nil, nil, fmt.Errorf("error getting stderr pipe for image copy process: %w", err)
+		return RegistryImageReference{}, fmt.Errorf("error getting stderr pipe for image copy process: %w", err)
 	}
 	defer stderr.Close()
 
 	stdin, err := cmd.StdinPipe()
 	if err != nil {
-		return nil, nil, fmt.Errorf("error getting stdin pipe for image copy process: %w", err)
+		return RegistryImageReference{}, fmt.Errorf("error getting stdin pipe for image copy process: %w", err)
 	}
 
 	stdinArguments := pullImageArgs{
@@ -725,44 +762,42 @@ func (svc *imageService) pullImageParent(ctx context.Context, imageName Registry
 	}
 
 	stdinArguments.Options.Progress = nil
+
 	if err := cmd.Start(); err != nil {
-		return nil, nil, err
+		return RegistryImageReference{}, err
 	}
+
 	if err := json.NewEncoder(stdin).Encode(&stdinArguments); err != nil {
 		stdin.Close()
+
 		if waitErr := cmd.Wait(); waitErr != nil {
-			return nil, nil, fmt.Errorf("%w: %w", waitErr, err)
+			return RegistryImageReference{}, fmt.Errorf("%w: %w", waitErr, err)
 		}
-		return nil, nil, fmt.Errorf("json encode to pipe failed: %w", err)
+
+		return RegistryImageReference{}, fmt.Errorf("json encode to pipe failed: %w", err)
 	}
+
 	stdin.Close()
 
 	resultChan := make(chan string)
-	var canonicalRef reference.Canonical
+
 	go func() {
 		defer func() {
 			close(resultChan) // Future reads, if any, will get "".
 		}()
 
 		decoder := json.NewDecoder(bufio.NewReader(stdout))
+
 		if progress != nil {
 			defer close(progress)
 		}
+
 		for decoder.More() {
 			var item pullImageOutputItem
 			if err := decoder.Decode(&item); err != nil {
 				break
 			}
 
-			named, err := reference.ParseNamed(item.Name)
-			if err != nil {
-				break
-			}
-			canonicalRef, err = reference.WithDigest(named, item.Digest)
-			if err != nil {
-				break
-			}
-
 			if item.Progress != nil && progress != nil {
 				progress <- *item.Progress
 			}
@@ -778,28 +813,25 @@ func (svc *imageService) pullImageParent(ctx context.Context, imageName Registry
 	errOutput, errReadAll := io.ReadAll(stderr)
 	if err := cmd.Wait(); err != nil {
 		if errReadAll == nil && len(errOutput) > 0 {
-			return nil, nil, fmt.Errorf("pull image: %s", string(errOutput))
+			return RegistryImageReference{}, fmt.Errorf("pull image: %s", string(errOutput))
 		}
-		return nil, nil, err
+
+		return RegistryImageReference{}, err
 	}
 
 	if result == "" {
-		return nil, nil, errors.New("pull child finished successfully but didn’t send a result")
+		return RegistryImageReference{}, errors.New("pull child finished successfully but didn’t send a result")
 	}
 
-	destRef, err := alltransports.ParseImageName(result)
+	canonicalRef, err := references.ParseRegistryImageReferenceFromOutOfProcessData(result)
 	if err != nil {
-		return nil, nil, err
+		return RegistryImageReference{}, err
 	}
 
-	if canonicalRef == nil {
-		return nil, nil, errors.New("canonical image reference cannot be nil")
-	}
-
-	return destRef, canonicalRef, nil
+	return canonicalRef, nil
 }
 
-func (svc *imageService) PullImage(ctx context.Context, imageName RegistryImageReference, options *ImageCopyOptions) (types.ImageReference, reference.Canonical, error) {
+func (svc *imageService) PullImage(ctx context.Context, imageName RegistryImageReference, options *ImageCopyOptions) (RegistryImageReference, error) {
 	if options.CgroupPull.UseNewCgroup {
 		return svc.pullImageParent(ctx, imageName, options.CgroupPull.ParentCgroup, options)
 	} else {
@@ -810,43 +842,68 @@ func (svc *imageService) PullImage(ctx context.Context, imageName RegistryImageR
 // pullImageImplementation is called in PullImage, both directly and inside pullImageChild.
 // NOTE: That means this code can run in a separate process, and it should not access any CRI-O global state.
 //
-// It returns a c/storage ImageReference for the destination.
-func pullImageImplementation(ctx context.Context, lookup *imageLookupService, store storage.Store, imageName RegistryImageReference, options *ImageCopyOptions) (types.ImageReference, reference.Canonical, error) {
-	srcSystemContext, srcRef, err := lookup.prepareReference(options.SourceCtx, imageName)
+// It returns a name@digest value referring to exactly the pulled image.
+func pullImageImplementation(ctx context.Context, lookup *imageLookupService, store storage.Store, imageName RegistryImageReference, options *ImageCopyOptions) (RegistryImageReference, error) {
+	srcRef, err := lookup.remoteImageReference(imageName)
 	if err != nil {
-		return nil, nil, err
+		return RegistryImageReference{}, err
 	}
+
+	srcSystemContext := types.SystemContext{}
+	if options.SourceCtx != nil {
+		srcSystemContext = *options.SourceCtx // A shallow copy
+	}
+
+	if secure := lookup.isSecureIndex(imageName.Registry()); !secure {
+		srcSystemContext.DockerInsecureSkipTLSVerify = types.OptionalBoolTrue
+	}
+
 	destRef, err := istorage.Transport.NewStoreReference(store, imageName.Raw(), "")
 	if err != nil {
-		return nil, nil, err
+		return RegistryImageReference{}, err
 	}
 
 	policy, err := signature.DefaultPolicy(options.SourceCtx)
 	if err != nil {
-		return nil, nil, err
+		return RegistryImageReference{}, err
 	}
+
 	policyContext, err := signature.NewPolicyContext(policy)
 	if err != nil {
-		return nil, nil, err
+		return RegistryImageReference{}, err
 	}
 
 	manifestBytes, err := copy.Image(ctx, policyContext, destRef, srcRef, &copy.Options{
-		SourceCtx:        srcSystemContext,
+		SourceCtx:        &srcSystemContext,
 		DestinationCtx:   options.DestinationCtx,
 		OciDecryptConfig: options.OciDecryptConfig,
 		ProgressInterval: options.ProgressInterval,
 		Progress:         options.Progress,
 	})
 	if err != nil {
-		return nil, nil, err
+		artifactManifestBytes, artifactErr := ociartifact.NewStore(store.GraphRoot(), &srcSystemContext).PullManifest(ctx, srcRef, &ociartifact.PullOptions{CopyOptions: &libimage.CopyOptions{
+			OciDecryptConfig: options.OciDecryptConfig,
+			Progress:         options.Progress,
+			RemoveSignatures: true, // signature is not supported for OCI layout dest
+		}})
+		if artifactErr != nil {
+			return RegistryImageReference{}, fmt.Errorf("unable to pull image or OCI artifact: pull image err: %w; artifact err: %w", err, artifactErr)
+		}
+
+		manifestBytes = artifactManifestBytes
 	}
 
-	canonicalRef, err := reference.WithDigest(reference.TrimNamed(imageName.Raw()), digest.FromBytes(manifestBytes))
+	manifestDigest, err := manifest.Digest(manifestBytes)
 	if err != nil {
-		return nil, nil, fmt.Errorf("create canonical reference: %w", err)
+		return RegistryImageReference{}, fmt.Errorf("digesting image: %w", err)
 	}
 
-	return destRef, canonicalRef, err
+	canonicalRef, err := reference.WithDigest(reference.TrimNamed(imageName.Raw()), manifestDigest)
+	if err != nil {
+		return RegistryImageReference{}, fmt.Errorf("create canonical reference: %w", err)
+	}
+
+	return references.RegistryImageReferenceFromRaw(canonicalRef), nil
 }
 
 func (svc *imageService) UntagImage(systemContext *types.SystemContext, name RegistryImageReference) error {
@@ -854,6 +911,7 @@ func (svc *imageService) UntagImage(systemContext *types.SystemContext, name Reg
 	if err != nil {
 		return err
 	}
+
 	_, img, err := svc.storageTransport.ResolveReference(unstableRef)
 	if err != nil {
 		return err
@@ -863,6 +921,7 @@ func (svc *imageService) UntagImage(systemContext *types.SystemContext, name Reg
 
 	nameString := name.Raw().String()
 	remainingNames := 0
+
 	for _, imgName := range img.Names {
 		if imgName != nameString {
 			remainingNames += 1
@@ -905,13 +964,12 @@ func (svc *imageLookupService) isSecureIndex(indexName string) bool {
 	addrs, err := net.LookupIP(host)
 	if err != nil {
 		ip := net.ParseIP(host)
-		if ip != nil {
-			addrs = []net.IP{ip}
-		}
-
 		// if ip == nil, then `host` is neither an IP nor it could be looked up,
 		// either because the index is unreachable, or because the index is behind an HTTP proxy.
 		// So, len(addrs) == 0 and we're not aborting.
+		if ip != nil {
+			addrs = []net.IP{ip}
+		}
 	}
 
 	// Try CIDR notation only if addrs has any elements, i.e. if `host`'s IP could be determined.
@@ -934,14 +992,17 @@ func (svc *imageService) HeuristicallyTryResolvingStringAsIDPrefix(heuristicInpu
 	if res, err := parseStorageImageID(heuristicInput); err == nil {
 		return &res // If it is already a full image ID, accept it.
 	}
+
 	if len(heuristicInput) >= minimumTruncatedIDLength {
 		if img, err := svc.store.Image(heuristicInput); err == nil && strings.HasPrefix(img.ID, heuristicInput) {
 			// It's a truncated version of the ID of an image that's present in local storage;
 			// we need to expand it.
 			res := storageImageIDFromImage(img)
+
 			return &res
 		}
 	}
+
 	return nil
 }
 
@@ -953,10 +1014,14 @@ func (svc *imageService) CandidatesForPotentiallyShortImageName(systemContext *t
 	if systemContext != nil {
 		sc = *systemContext // A shallow copy
 	}
-	disabled := types.ShortNameModeDisabled
-	sc.ShortNameMode = &disabled
+
 	resolved, err := shortnames.Resolve(&sc, imageName)
 	if err != nil {
+		// Error is not very clear in this context, and unfortunately is also not a variable.
+		if strings.Contains(err.Error(), "short-name resolution enforced but cannot prompt without a TTY") {
+			return nil, fmt.Errorf("short name mode is enforcing, but image name %s returns ambiguous list", imageName)
+		}
+
 		return nil, err
 	}
 
@@ -981,18 +1046,22 @@ func (svc *imageService) CandidatesForPotentiallyShortImageName(systemContext *t
 func GetImageService(ctx context.Context, store storage.Store, storageTransport StorageTransport, serverConfig *config.Config) (ImageServer, error) {
 	if store == nil {
 		var err error
+
 		storeOpts, err := storage.DefaultStoreOptions()
 		if err != nil {
 			return nil, err
 		}
+
 		store, err = storage.GetStore(storeOpts)
 		if err != nil {
 			return nil, err
 		}
 	}
+
 	if storageTransport == nil {
 		storageTransport = nativeStorageTransport{}
 	}
+
 	ils := &imageLookupService{
 		DefaultTransport:      serverConfig.DefaultTransport,
 		IndexConfigs:          make(map[string]*indexInfo),
@@ -1002,6 +1071,7 @@ func GetImageService(ctx context.Context, store storage.Store, storageTransport
 	if serverConfig.PauseImage != "" {
 		serverConfig.PinnedImages = append(serverConfig.PinnedImages, serverConfig.PauseImage)
 	}
+
 	is := &imageService{
 		lookup:               ils,
 		store:                store,
@@ -1012,6 +1082,10 @@ func GetImageService(ctx context.Context, store storage.Store, storageTransport
 		regexForPinnedImages: CompileRegexpsForPinnedImages(serverConfig.PinnedImages),
 	}
 
+	if len(serverConfig.InsecureRegistries) > 0 {
+		log.Warnf(ctx, "Insecure registries option is deprecated and will not have any effect in a future release")
+	}
+
 	serverConfig.InsecureRegistries = append(serverConfig.InsecureRegistries, "127.0.0.0/8")
 	// Split --insecure-registry into CIDR and registry-specific settings.
 	for _, r := range serverConfig.InsecureRegistries {
@@ -1060,6 +1134,7 @@ func FilterPinnedImage(image string, pinnedImages []*regexp.Regexp) bool {
 			return true
 		}
 	}
+
 	return false
 }
 
@@ -1067,8 +1142,10 @@ func FilterPinnedImage(image string, pinnedImages []*regexp.Regexp) bool {
 // list of pinned images.
 func CompileRegexpsForPinnedImages(patterns []string) []*regexp.Regexp {
 	regexps := make([]*regexp.Regexp, 0, len(patterns))
+
 	for _, pattern := range patterns {
 		var re *regexp.Regexp
+
 		switch {
 		case strings.HasPrefix(pattern, "*") && strings.HasSuffix(pattern, "*"):
 			// keyword pattern
@@ -1082,6 +1159,7 @@ func CompileRegexpsForPinnedImages(patterns []string) []*regexp.Regexp {
 			// exact pattern
 			re = regexp.MustCompile("(?i)^" + regexp.QuoteMeta(pattern) + "$")
 		}
+
 		regexps = append(regexps, re)
 	}
 
