@@ -9,10 +9,10 @@ import (
 	istorage "github.com/containers/image/v5/storage"
 	"github.com/containers/image/v5/types"
 	cs "github.com/containers/storage"
-	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	digest "github.com/opencontainers/go-digest"
+	"go.uber.org/mock/gomock"
 
 	"github.com/cri-o/cri-o/internal/mockutils"
 	"github.com/cri-o/cri-o/internal/storage"
@@ -29,7 +29,6 @@ var _ = t.Describe("Image", func() {
 		testDockerRegistry                  = "docker.io"
 		testQuayRegistry                    = "quay.io"
 		testRedHatRegistry                  = "registry.access.redhat.com"
-		testFedoraRegistry                  = "registry.fedoraproject.org"
 		testImageName                       = "image"
 		testImageAlias                      = "image-for-testing"
 		testImageAliasResolved              = "registry.crio.test.com/repo"
@@ -172,6 +171,7 @@ var _ = t.Describe("Image", func() {
 			for _, ref := range refs {
 				names = append(names, ref.StringForOutOfProcessConsumptionOnly())
 			}
+
 			return names
 		}
 
@@ -182,7 +182,8 @@ var _ = t.Describe("Image", func() {
 			// When
 			refs, err := sut.CandidatesForPotentiallyShortImageName(
 				&types.SystemContext{
-					SystemRegistriesConfPath: "../../test/registries.conf",
+					SystemRegistriesConfPath:    "../../test/registries.conf",
+					SystemRegistriesConfDirPath: t.MustTempDir("registries.conf.d"),
 				},
 				testImageName,
 			)
@@ -192,7 +193,6 @@ var _ = t.Describe("Image", func() {
 			Expect(refsToNames(refs)).To(Equal([]string{
 				testQuayRegistry + "/" + testImageName + ":latest",
 				testRedHatRegistry + "/" + testImageName + ":latest",
-				testFedoraRegistry + "/" + testImageName + ":latest",
 				testDockerRegistry + "/library/" + testImageName + ":latest",
 			}))
 		})
@@ -237,7 +237,8 @@ var _ = t.Describe("Image", func() {
 			// When
 			refs, err := sut.CandidatesForPotentiallyShortImageName(
 				&types.SystemContext{
-					SystemRegistriesConfPath: "../../test/registries.conf",
+					SystemRegistriesConfPath:    "../../test/registries.conf",
+					SystemRegistriesConfDirPath: t.MustTempDir("registries.conf.d"),
 				},
 				testImageWithTagAndDigest,
 			)
@@ -246,7 +247,6 @@ var _ = t.Describe("Image", func() {
 			Expect(refsToNames(refs)).To(Equal([]string{
 				testQuayRegistry + "/" + testImageName + "@sha256:" + testSHA256,
 				testRedHatRegistry + "/" + testImageName + "@sha256:" + testSHA256,
-				testFedoraRegistry + "/" + testImageName + "@sha256:" + testSHA256,
 				testDockerRegistry + "/library/" + testImageName + "@sha256:" + testSHA256,
 			}))
 		})
@@ -295,7 +295,8 @@ var _ = t.Describe("Image", func() {
 			// When
 			refs, err := sut.CandidatesForPotentiallyShortImageName(
 				&types.SystemContext{
-					SystemRegistriesConfPath: "/dev/null",
+					SystemRegistriesConfPath:    "/dev/null",
+					SystemRegistriesConfDirPath: t.MustTempDir("registries.conf.d"),
 				},
 				testImageName,
 			)
@@ -577,90 +578,102 @@ var _ = t.Describe("Image", func() {
 			Expect(err).ToNot(HaveOccurred())
 
 			// When
-			res, _, err := sut.PullImage(context.Background(), imageRef, &storage.ImageCopyOptions{
+			res, err := sut.PullImage(context.Background(), imageRef, &storage.ImageCopyOptions{
 				SourceCtx: &types.SystemContext{SignaturePolicyPath: "/not-existing"},
 			})
 
 			// Then
 			Expect(err).To(HaveOccurred())
-			Expect(res).To(BeNil())
+			Expect(res).To(Equal(storage.RegistryImageReference{}))
 		})
 
 		It("should fail on copy image", func() {
 			// Given
+			mockutils.InOrder(
+				storeMock.EXPECT().GraphRoot().Return(""),
+			)
 			imageRef, err := references.ParseRegistryImageReferenceFromOutOfProcessData("localhost/busybox:latest")
 			Expect(err).ToNot(HaveOccurred())
 
 			// When
-			res, _, err := sut.PullImage(context.Background(), imageRef, &storage.ImageCopyOptions{
+			res, err := sut.PullImage(context.Background(), imageRef, &storage.ImageCopyOptions{
 				SourceCtx: &types.SystemContext{SignaturePolicyPath: "../../test/policy.json"},
 			})
 
 			// Then
 			Expect(err).To(HaveOccurred())
-			Expect(res).To(BeNil())
+			Expect(res).To(Equal(storage.RegistryImageReference{}))
 		})
 
 		It("should fail on canonical copy image", func() {
 			// Given
+			mockutils.InOrder(
+				storeMock.EXPECT().GraphRoot().Return(""),
+			)
 			imageRef, err := references.ParseRegistryImageReferenceFromOutOfProcessData("localhost/busybox@sha256:" + testSHA256)
 			Expect(err).ToNot(HaveOccurred())
 
 			// When
-			res, _, err := sut.PullImage(context.Background(), imageRef, &storage.ImageCopyOptions{
+			res, err := sut.PullImage(context.Background(), imageRef, &storage.ImageCopyOptions{
 				SourceCtx: &types.SystemContext{SignaturePolicyPath: "../../test/policy.json"},
 			})
 
 			// Then
 			Expect(err).To(HaveOccurred())
-			Expect(res).To(BeNil())
+			Expect(res).To(Equal(storage.RegistryImageReference{}))
 		})
 
 		It("should fail on cancelled context", func() {
 			// Given
+			mockutils.InOrder(
+				storeMock.EXPECT().GraphRoot().Return(""),
+			)
 			imageRef, err := references.ParseRegistryImageReferenceFromOutOfProcessData("localhost/busybox:latest")
 			Expect(err).ToNot(HaveOccurred())
 
 			// When
 			ctx, cancel := context.WithCancel(context.Background())
 			cancel()
-			res, _, err := sut.PullImage(ctx, imageRef, &storage.ImageCopyOptions{
+			res, err := sut.PullImage(ctx, imageRef, &storage.ImageCopyOptions{
 				SourceCtx: &types.SystemContext{SignaturePolicyPath: "../../test/policy.json"},
 			})
 
 			// Then
 			Expect(err).To(HaveOccurred())
 			Expect(err.Error()).To(ContainSubstring("context canceled"))
-			Expect(res).To(BeNil())
+			Expect(res).To(Equal(storage.RegistryImageReference{}))
 		})
 
 		It("should fail on timed out context", func() {
 			// Given
+			mockutils.InOrder(
+				storeMock.EXPECT().GraphRoot().Return(""),
+			)
 			imageRef, err := references.ParseRegistryImageReferenceFromOutOfProcessData("localhost/busybox:latest")
 			Expect(err).ToNot(HaveOccurred())
 
 			// When
 			ctx, cancel := context.WithTimeout(context.Background(), 0)
 			defer cancel()
-			res, _, err := sut.PullImage(ctx, imageRef, &storage.ImageCopyOptions{
+			res, err := sut.PullImage(ctx, imageRef, &storage.ImageCopyOptions{
 				SourceCtx: &types.SystemContext{SignaturePolicyPath: "../../test/policy.json"},
 			})
 
 			// Then
 			Expect(err).To(HaveOccurred())
 			Expect(err.Error()).To(ContainSubstring("context deadline exceeded"))
-			Expect(res).To(BeNil())
+			Expect(res).To(Equal(storage.RegistryImageReference{}))
 		})
 	})
 
 	t.Describe("CompileRegexpsForPinnedImages", func() {
 		It("should return regexps for exact patterns", func() {
-			patterns := []string{"quay.io/crio/pause:latest", "docker.io/crio/sandbox:latest", "registry.k8s.io/pause:3.10"}
+			patterns := []string{"quay.io/crio/pause:latest", "docker.io/crio/sandbox:latest", "registry.k8s.io/pause:3.10.1"}
 			regexps := storage.CompileRegexpsForPinnedImages(patterns)
 			Expect(regexps).To(HaveLen(len(patterns)))
 			Expect(regexps[0].MatchString("quay.io/crio/pause:latest")).To(BeTrue())
 			Expect(regexps[1].MatchString("docker.io/crio/sandbox:latest")).To(BeTrue())
-			Expect(regexps[2].MatchString("registry.k8s.io/pause:3.10")).To(BeTrue())
+			Expect(regexps[2].MatchString("registry.k8s.io/pause:3.10.1")).To(BeTrue())
 		})
 
 		It("should return regexps for keyword patterns", func() {
