@@ -1,5 +1,4 @@
 //go:build !linux && !freebsd && !windows
-// +build !linux,!freebsd,!windows
 
 package config
 
@@ -15,7 +14,7 @@ const (
 	// ImageVolumesBind option is for using bind mounted volumes
 	ImageVolumesBind ImageVolumesType = "invalid ImageVolumesBind"
 	// DefaultPauseImage is default pause image
-	DefaultPauseImage string = "registry.k8s.io/pause:3.10"
+	DefaultPauseImage string = "registry.k8s.io/pause:3.10.1"
 )
 
 func selinuxEnabled() bool {
@@ -26,3 +25,7 @@ func selinuxEnabled() bool {
 func checkKernelRROMountSupport() error {
 	return errdefs.ErrNotImplemented
 }
+
+func (c *RuntimeConfig) ValidatePinnsPath(executable string) error {
+	return nil
+}
