@@ -1,10 +1,10 @@
 package server
 
 import (
+	"context"
 	"errors"
 
 	metadata "github.com/checkpoint-restore/checkpointctl/lib"
-	"golang.org/x/net/context"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
@@ -15,32 +15,32 @@ import (
 
 // CheckpointContainer checkpoints a container.
 func (s *Server) CheckpointContainer(ctx context.Context, req *types.CheckpointContainerRequest) (*types.CheckpointContainerResponse, error) {
-	if !s.config.RuntimeConfig.CheckpointRestore() {
+	if !s.config.CheckpointRestore() {
 		return nil, errors.New("checkpoint/restore support not available")
 	}
 
-	_, err := s.GetContainerFromShortID(ctx, req.ContainerId)
+	_, err := s.GetContainerFromShortID(ctx, req.GetContainerId())
 	if err != nil {
-		return nil, status.Errorf(codes.NotFound, "could not find container %q: %v", req.ContainerId, err)
+		return nil, status.Errorf(codes.NotFound, "could not find container %q: %v", req.GetContainerId(), err)
 	}
 
-	log.Infof(ctx, "Checkpointing container: %s", req.ContainerId)
+	log.Infof(ctx, "Checkpointing container: %s", req.GetContainerId())
 	config := &metadata.ContainerConfig{
-		ID: req.ContainerId,
+		ID: req.GetContainerId(),
 	}
 	opts := &lib.ContainerCheckpointOptions{
-		TargetFile: req.Location,
+		TargetFile: req.GetLocation(),
 		// For the forensic container checkpointing use case we
 		// keep the container running after checkpointing it.
 		KeepRunning: true,
 	}
 
-	_, err = s.ContainerServer.ContainerCheckpoint(ctx, config, opts)
+	_, err = s.ContainerCheckpoint(ctx, config, opts)
 	if err != nil {
 		return nil, err
 	}
 
-	log.Infof(ctx, "Checkpointed container: %s", req.ContainerId)
+	log.Infof(ctx, "Checkpointed container: %s", req.GetContainerId())
 
 	return &types.CheckpointContainerResponse{}, nil
 }
