@@ -9,11 +9,11 @@ import (
 	criu "github.com/checkpoint-restore/go-criu/v7/utils"
 	"github.com/containers/storage/pkg/archive"
 	"github.com/containers/storage/pkg/unshare"
-	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	v1 "github.com/opencontainers/image-spec/specs-go/v1"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
+	"go.uber.org/mock/gomock"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 	kubetypes "k8s.io/kubelet/pkg/types"
 
@@ -78,7 +78,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 
@@ -103,7 +103,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 			// Then
@@ -126,7 +126,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 			// Then
@@ -161,7 +161,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 			// Then
@@ -199,7 +199,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 
@@ -239,7 +239,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 
@@ -285,7 +285,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 
@@ -334,7 +334,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 
@@ -342,59 +342,6 @@ var _ = t.Describe("ContainerRestore", func() {
 			Expect(err.Error()).To(Equal(`failed to read "io.kubernetes.cri-o.Annotations": unexpected end of JSON input`))
 		})
 	})
-	t.Describe("ContainerRestore from archive into new pod", func() {
-		It("should fail with 'PodSandboxId should not be empty'", func() {
-			// Given
-			addContainerAndSandbox()
-			testContainer.SetStateAndSpoofPid(&oci.ContainerState{
-				State: specs.State{Status: oci.ContainerStateRunning},
-			})
-
-			err := os.WriteFile(
-				"spec.dump",
-				[]byte(
-					`{"annotations":{"io.kubernetes.cri-o.Metadata"`+
-						`:"{\"name\":\"container-to-restore\"}",`+
-						`"io.kubernetes.cri-o.Annotations": "{\"name\":\"NAME\"}",`+
-						`"io.kubernetes.cri-o.Labels": "{\"io.kubernetes.container.name\":\"counter\"}"}}`),
-				0o644,
-			)
-			Expect(err).ToNot(HaveOccurred())
-			defer os.RemoveAll("spec.dump")
-			err = os.WriteFile("config.dump", []byte(`{"rootfsImageName": "image"}`), 0o644)
-			Expect(err).ToNot(HaveOccurred())
-			defer os.RemoveAll("config.dump")
-			outFile, err := os.Create("archive.tar")
-			Expect(err).ToNot(HaveOccurred())
-			defer outFile.Close()
-			input, err := archive.TarWithOptions(".", &archive.TarOptions{
-				Compression:      archive.Uncompressed,
-				IncludeSourceDir: true,
-				IncludeFiles:     []string{"spec.dump", "config.dump"},
-			})
-			Expect(err).ToNot(HaveOccurred())
-			defer os.RemoveAll("archive.tar")
-			_, err = io.Copy(outFile, input)
-			Expect(err).ToNot(HaveOccurred())
-			containerConfig := &types.ContainerConfig{
-				Metadata: &types.ContainerMetadata{Name: "name"},
-				Image: &types.ImageSpec{
-					Image: "archive.tar",
-				},
-			}
-			// When
-
-			_, err = sut.CRImportCheckpoint(
-				context.Background(),
-				containerConfig,
-				"",
-				"",
-			)
-
-			// Then
-			Expect(err.Error()).To(Equal(`PodSandboxId should not be empty`))
-		})
-	})
 	t.Describe("ContainerRestore from archive into new pod", func() {
 		images := []struct {
 			config string
@@ -529,6 +476,7 @@ var _ = t.Describe("ContainerRestore", func() {
 						),
 					runtimeServerMock.EXPECT().StartContainer(gomock.Any()).
 						Return(emptyDir, nil),
+					storeMock.EXPECT().GraphRoot().Return(""),
 				)
 
 				// When
@@ -536,7 +484,7 @@ var _ = t.Describe("ContainerRestore", func() {
 				_, err = sut.CRImportCheckpoint(
 					context.Background(),
 					containerConfig,
-					"",
+					testSandbox,
 					"new-sandbox-id",
 				)
 
@@ -548,6 +496,7 @@ var _ = t.Describe("ContainerRestore", func() {
 	t.Describe("ContainerRestore from OCI archive", func() {
 		It("should fail because archive does not exist", func() {
 			// Given
+			addContainerAndSandbox()
 			size := uint64(100)
 			checkpointImageName, err := references.ParseRegistryImageReferenceFromOutOfProcessData("localhost/checkpoint-image:tag1")
 			Expect(err).ToNot(HaveOccurred())
@@ -585,7 +534,7 @@ var _ = t.Describe("ContainerRestore", func() {
 			_, err = sut.CRImportCheckpoint(
 				context.Background(),
 				containerConfig,
-				"",
+				testSandbox,
 				"",
 			)
 
