@@ -14,16 +14,18 @@ import (
 func (s *Server) ContainerStats(ctx context.Context, req *types.ContainerStatsRequest) (*types.ContainerStatsResponse, error) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	container, err := s.GetContainerFromShortID(ctx, req.ContainerId)
+
+	container, err := s.GetContainerFromShortID(ctx, req.GetContainerId())
 	if err != nil {
 		return nil, err
 	}
+
 	sb := s.GetSandbox(container.Sandbox())
 	if sb == nil {
 		return nil, fmt.Errorf("unable to get stats for container %s: sandbox %s not found", container.ID(), container.Sandbox())
 	}
 
 	return &types.ContainerStatsResponse{
-		Stats: s.ContainerServer.StatsForContainer(container, sb),
+		Stats: s.StatsForContainer(container, sb),
 	}, nil
 }
