@@ -1,7 +1,8 @@
 package server
 
 import (
-	"golang.org/x/net/context"
+	"context"
+
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
@@ -12,6 +13,7 @@ func (s *Server) RuntimeConfig(_ context.Context, req *types.RuntimeConfigReques
 			CgroupDriver: s.getCgroupDriver(),
 		},
 	}
+
 	return resp, nil
 }
 
@@ -19,5 +21,6 @@ func (s *Server) getCgroupDriver() types.CgroupDriver {
 	if s.config.CgroupManager().IsSystemd() {
 		return types.CgroupDriver_SYSTEMD
 	}
+
 	return types.CgroupDriver_CGROUPFS
 }
