@@ -27,9 +27,9 @@ var _ = t.Describe("Status", func() {
 			// Then
 			Expect(err).ToNot(HaveOccurred())
 			Expect(response).NotTo(BeNil())
-			Expect(len(response.Status.Conditions)).To(BeEquivalentTo(2))
-			for _, condition := range response.Status.Conditions {
-				Expect(condition.Status).To(BeTrue())
+			Expect(len(response.GetStatus().GetConditions())).To(BeEquivalentTo(2))
+			for _, condition := range response.GetStatus().GetConditions() {
+				Expect(condition.GetStatus()).To(BeTrue())
 			}
 		})
 
@@ -41,9 +41,9 @@ var _ = t.Describe("Status", func() {
 			// Then
 			Expect(err).ToNot(HaveOccurred())
 			Expect(response).NotTo(BeNil())
-			Expect(len(response.Status.Conditions)).To(BeEquivalentTo(2))
-			for _, condition := range response.Status.Conditions {
-				Expect(condition.Status).To(BeTrue())
+			Expect(len(response.GetStatus().GetConditions())).To(BeEquivalentTo(2))
+			for _, condition := range response.GetStatus().GetConditions() {
+				Expect(condition.GetStatus()).To(BeTrue())
 			}
 		})
 
@@ -55,7 +55,7 @@ var _ = t.Describe("Status", func() {
 			// Then
 			Expect(err).ToNot(HaveOccurred())
 			Expect(response).NotTo(BeNil())
-			Expect(response.Info).NotTo(BeNil())
+			Expect(response.GetInfo()).NotTo(BeNil())
 		})
 	})
 })
