@@ -1,14 +1,15 @@
 package server
 
 import (
-	"golang.org/x/net/context"
+	"context"
+
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // ListPodSandboxMetrics lists all pod sandbox metrics.
 func (s *Server) ListPodSandboxMetrics(ctx context.Context, req *types.ListPodSandboxMetricsRequest) (*types.ListPodSandboxMetricsResponse, error) {
-	sboxList := s.ContainerServer.ListSandboxes()
-	metricsList := s.ContainerServer.MetricsForPodSandboxList(sboxList)
+	sboxList := s.ListSandboxes()
+	metricsList := s.MetricsForPodSandboxList(sboxList)
 	responseMetricsList := make([]*types.PodSandboxMetrics, 0, len(metricsList))
 
 	for _, metrics := range metricsList {
@@ -16,10 +17,10 @@ func (s *Server) ListPodSandboxMetrics(ctx context.Context, req *types.ListPodSa
 			responseMetricsList = append(responseMetricsList, current)
 		} else {
 			// Iterate over container metrics within each PodSandboxMetrics.
-			containerMetricsList := metrics.GetMetric().ContainerMetrics
+			containerMetricsList := metrics.GetMetric().GetContainerMetrics()
 			for _, containerMetrics := range containerMetricsList {
 				containerPodSandboxMetrics := &types.PodSandboxMetrics{
-					PodSandboxId:     metrics.GetMetric().PodSandboxId,
+					PodSandboxId:     metrics.GetMetric().GetPodSandboxId(),
 					ContainerMetrics: []*types.ContainerMetrics{containerMetrics},
 				}
 				responseMetricsList = append(responseMetricsList, containerPodSandboxMetrics)
