@@ -1,10 +1,10 @@
 package server
 
 import (
+	"context"
 	"errors"
 	"fmt"
 
-	"golang.org/x/net/context"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
@@ -17,24 +17,28 @@ func (s *Server) StopPodSandbox(ctx context.Context, req *types.StopPodSandboxRe
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
 	// platform dependent call
-	log.Infof(ctx, "Stopping pod sandbox: %s", req.PodSandboxId)
-	sb, err := s.getPodSandboxFromRequest(ctx, req.PodSandboxId)
+	log.Infof(ctx, "Stopping pod sandbox: %s", req.GetPodSandboxId())
+
+	sb, err := s.getPodSandboxFromRequest(ctx, req.GetPodSandboxId())
 	if err != nil {
 		if errors.Is(err, sandbox.ErrIDEmpty) {
 			return nil, err
 		}
+
 		if errors.Is(err, errSandboxNotCreated) {
-			return nil, fmt.Errorf("StopPodSandbox failed as the sandbox is not created: %s", req.PodSandboxId)
+			return nil, fmt.Errorf("StopPodSandbox failed as the sandbox is not created: %s", req.GetPodSandboxId())
 		}
 
 		// If the sandbox isn't found we just return an empty response to adhere
 		// the CRI interface which expects to not error out in not found
 		// cases.
 
-		log.Warnf(ctx, "Could not get sandbox %s, it's probably been stopped already: %v", req.PodSandboxId, err)
-		log.Debugf(ctx, "StopPodSandboxResponse %s", req.PodSandboxId)
+		log.Warnf(ctx, "Could not get sandbox %s, it's probably been stopped already: %v", req.GetPodSandboxId(), err)
+		log.Debugf(ctx, "StopPodSandboxResponse %s", req.GetPodSandboxId())
+
 		return &types.StopPodSandboxResponse{}, nil
 	}
+
 	if err := s.stopPodSandbox(ctx, sb); err != nil {
 		return nil, err
 	}
