@@ -3,7 +3,6 @@ package server
 import (
 	"context"
 	"crypto/tls"
-	"crypto/x509"
 	"errors"
 	"fmt"
 	"net"
@@ -19,7 +18,6 @@ import (
 	"github.com/containers/storage/pkg/idtools"
 	storageTypes "github.com/containers/storage/types"
 	"github.com/fsnotify/fsnotify"
-	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
@@ -27,6 +25,7 @@ import (
 	"k8s.io/kubelet/pkg/cri/streaming"
 	kubetypes "k8s.io/kubelet/pkg/types"
 
+	"github.com/cri-o/cri-o/internal/cert"
 	"github.com/cri-o/cri-o/internal/config/seccomp"
 	"github.com/cri-o/cri-o/internal/hostport"
 	"github.com/cri-o/cri-o/internal/lib"
@@ -34,18 +33,19 @@ import (
 	"github.com/cri-o/cri-o/internal/log"
 	nriIf "github.com/cri-o/cri-o/internal/nri"
 	"github.com/cri-o/cri-o/internal/oci"
+	"github.com/cri-o/cri-o/internal/ociartifact"
 	"github.com/cri-o/cri-o/internal/resourcestore"
 	"github.com/cri-o/cri-o/internal/runtimehandlerhooks"
 	"github.com/cri-o/cri-o/internal/signals"
 	"github.com/cri-o/cri-o/internal/storage"
 	"github.com/cri-o/cri-o/internal/version"
+	"github.com/cri-o/cri-o/internal/watchdog"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 	"github.com/cri-o/cri-o/server/metrics"
 	"github.com/cri-o/cri-o/utils"
 )
 
 const (
-	certRefreshInterval            = time.Minute * 5
 	rootlessEnvName                = "_CRIO_ROOTLESS"
 	irqBalanceConfigRestoreDisable = "disable"
 	debounceDuration               = 200 * time.Millisecond
@@ -66,7 +66,7 @@ type StreamService struct {
 // Server implements the RuntimeService and ImageService.
 type Server struct {
 	config          libconfig.Config
-	stream          StreamService
+	stream          *StreamService
 	hostportManager hostport.HostPortManager
 
 	*lib.ContainerServer
@@ -92,6 +92,11 @@ type Server struct {
 
 	// NRI runtime interface
 	nri *nriAPI
+	// hooksRetriever allows getting the runtime hooks for the sandboxes.
+	hooksRetriever *runtimehandlerhooks.HooksRetriever
+
+	types.UnsafeImageServiceServer
+	types.UnsafeRuntimeServiceServer
 }
 
 // pullArguments are used to identify a pullOperation via an input image name and
@@ -110,49 +115,13 @@ type pullOperation struct {
 	// wg allows for Goroutines trying to pull the same image to wait until the
 	// currently running pull operation has finished.
 	wg sync.WaitGroup
-	// imageRef is the reference of the actually pulled image which will differ
-	// from the input if it was a short name (e.g., alpine).
-	imageRef string
+	// imageRef is the reference of the actually pulled image; it is always
+	// in a full repo@digest format, resolving short names and tags
+	imageRef storage.RegistryImageReference
 	// err is the error indicating if the pull operation has succeeded or not.
 	err error
 }
 
-type certConfigCache struct {
-	config  *tls.Config
-	expires time.Time
-
-	tlsCert string
-	tlsKey  string
-	tlsCA   string
-}
-
-// GetConfigForClient gets the tlsConfig for the streaming server.
-// This allows the certs to be swapped, without shutting down crio.
-func (cc *certConfigCache) GetConfigForClient(hello *tls.ClientHelloInfo) (*tls.Config, error) {
-	if cc.config != nil && time.Now().Before(cc.expires) {
-		return cc.config, nil
-	}
-	config := new(tls.Config)
-	cert, err := tls.LoadX509KeyPair(cc.tlsCert, cc.tlsKey)
-	if err != nil {
-		return nil, err
-	}
-	config.Certificates = []tls.Certificate{cert}
-	if cc.tlsCA != "" {
-		caBytes, err := os.ReadFile(cc.tlsCA)
-		if err != nil {
-			return nil, fmt.Errorf("read TLS CA file: %w", err)
-		}
-		certPool := x509.NewCertPool()
-		certPool.AppendCertsFromPEM(caBytes)
-		config.ClientCAs = certPool
-		config.ClientAuth = tls.RequireAndVerifyClientCert
-	}
-	cc.config = config
-	cc.expires = time.Now().Add(certRefreshInterval)
-	return config, nil
-}
-
 // StopStreamServer stops the stream server.
 func (s *Server) StopStreamServer() error {
 	return s.stream.streamServer.Stop()
@@ -185,49 +154,76 @@ func (s *Server) getPortForward(req *types.PortForwardRequest) (*types.PortForwa
 func (s *Server) restore(ctx context.Context) []storage.StorageImageID {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
+
 	containersAndTheirImages := map[string]storage.StorageImageID{}
-	containers, err := s.Store().Containers()
+
+	containers, err := s.ContainerServer.Store().Containers()
 	if err != nil && !errors.Is(err, os.ErrNotExist) {
 		log.Warnf(ctx, "Could not read containers and sandboxes: %v", err)
 	}
+
 	pods := map[string]*storage.RuntimeContainerMetadata{}
 	podContainers := map[string]*storage.RuntimeContainerMetadata{}
 	names := map[string][]string{}
 	deletedPods := map[string]*sandbox.Sandbox{}
+
 	for i := range containers {
-		metadata, err2 := s.StorageRuntimeServer().GetContainerMetadata(containers[i].ID)
+		metadata, err2 := s.ContainerServer.StorageRuntimeServer().GetContainerMetadata(containers[i].ID)
 		if err2 != nil {
 			log.Warnf(ctx, "Error parsing metadata for %s: %v, ignoring", containers[i].ID, err2)
+
 			continue
 		}
+
 		if !storage.IsCrioContainer(&metadata) {
 			log.Debugf(ctx, "Container %s determined to not be a CRI-O container or sandbox", containers[i].ID)
+
 			continue
 		}
+
 		names[containers[i].ID] = containers[i].Names
+
 		if metadata.Pod {
 			pods[containers[i].ID] = &metadata
 		} else {
 			podContainers[containers[i].ID] = &metadata
+
 			imageID, err := storage.ParseStorageImageIDFromOutOfProcessData(containers[i].ImageID)
 			if err != nil {
 				log.Warnf(ctx, "Error parsing image ID %q of container %q: %v, ignoring", containers[i].ImageID, containers[i].ID, err)
+
 				continue
 			}
+
 			containersAndTheirImages[containers[i].ID] = imageID
 		}
 	}
 
 	// Go through all the pods and check if it can be restored. If an error occurs, delete the pod and any containers
 	// associated with it. Release the pod and container names as well.
+	knownPods := []*sandbox.Sandbox{}
+
 	for sbID := range pods {
 		sb, err := s.LoadSandbox(ctx, sbID)
+		// If we were able to restore a sandbox, add the pod id to the list of deletedPods, to be able to call CNI DEL
+		// on the sandbox network. Otherwise, exclude pods for which we weren't able to restore a sandbox from the
+		// knownPods list so that any potential stale resource associated to them is cleaned up in the network plugin GC
+		if sb != nil {
+			knownPods = append(knownPods, sb)
+
+			if err != nil {
+				deletedPods[sbID] = sb
+			}
+		}
+
 		if err == nil {
 			continue
 		}
+
 		log.Warnf(ctx, "Could not restore sandbox %s: %v", sbID, err)
+
 		for _, n := range names[sbID] {
-			if err := s.Store().DeleteContainer(n); err != nil && !errors.Is(err, storageTypes.ErrNotAContainer) {
+			if err := s.ContainerServer.Store().DeleteContainer(n); err != nil && !errors.Is(err, storageTypes.ErrNotAContainer) {
 				log.Warnf(ctx, "Unable to delete container %s: %v", n, err)
 			}
 			// Release the infra container name and the pod name for future use
@@ -239,12 +235,14 @@ func (s *Server) restore(ctx context.Context) []storage.StorageImageID {
 		}
 		// Go through the containers and delete any container that was under the deleted pod
 		log.Warnf(ctx, "Deleting all containers under sandbox %s since it could not be restored", sbID)
+
 		for k, v := range podContainers {
 			if v.PodID != sbID {
 				continue
 			}
+
 			for _, n := range names[k] {
-				if err := s.Store().DeleteContainer(n); err != nil && !errors.Is(err, storageTypes.ErrNotAContainer) {
+				if err := s.ContainerServer.Store().DeleteContainer(n); err != nil && !errors.Is(err, storageTypes.ErrNotAContainer) {
 					log.Warnf(ctx, "Unable to delete container %s: %v", n, err)
 				}
 				// Release the container name for future use
@@ -254,11 +252,6 @@ func (s *Server) restore(ctx context.Context) []storage.StorageImageID {
 			// causing a useless debug message.
 			delete(podContainers, k)
 		}
-		// Add the pod id to the list of deletedPods, to be able to call CNI DEL on the sandbox network.
-		// Unfortunately, if we weren't able to restore a sandbox, then there's little that can be done
-		if sb != nil {
-			deletedPods[sbID] = sb
-		}
 	}
 
 	// Go through all the containers and check if it can be restored. If an error occurs, delete the container and
@@ -267,11 +260,14 @@ func (s *Server) restore(ctx context.Context) []storage.StorageImageID {
 		err := s.LoadContainer(ctx, containerID)
 		if err == nil || errors.Is(err, lib.ErrIsNonCrioContainer) {
 			delete(containersAndTheirImages, containerID)
+
 			continue
 		}
+
 		log.Warnf(ctx, "Could not restore container %s: %v", containerID, err)
+
 		for _, n := range names[containerID] {
-			if err := s.Store().DeleteContainer(n); err != nil && !errors.Is(err, storageTypes.ErrNotAContainer) {
+			if err := s.ContainerServer.Store().DeleteContainer(n); err != nil && !errors.Is(err, storageTypes.ErrNotAContainer) {
 				log.Warnf(ctx, "Unable to delete container %s: %v", n, err)
 			}
 			// Release the container name
@@ -279,14 +275,23 @@ func (s *Server) restore(ctx context.Context) []storage.StorageImageID {
 		}
 	}
 
+	// Cleanup any potential stale network resources not associated to any
+	// pod known to us using CNI GC
+	err = s.networkGC(context.Background(), knownPods)
+	if err != nil {
+		log.Errorf(ctx, "Garbage collect stale network resources during server startup failed: %v", err)
+	}
+
 	// Cleanup the deletedPods in the networking plugin
 	wipeResourceCleaner := resourcestore.NewResourceCleaner()
+
 	for _, sb := range deletedPods {
 		cleanupFunc := func() error {
 			err := s.networkStop(context.Background(), sb)
 			if err == nil {
 				log.Infof(ctx, "Successfully cleaned up network for pod %s", sb.ID())
 			}
+
 			return err
 		}
 		wipeResourceCleaner.Add(ctx, "cleanup sandbox network", cleanupFunc)
@@ -305,8 +310,10 @@ func (s *Server) restore(ctx context.Context) []storage.StorageImageID {
 		ips, err := s.getSandboxIPs(ctx, sb)
 		if err != nil {
 			log.Warnf(ctx, "Could not restore sandbox IP for %v: %v", sb.ID(), err)
+
 			continue
 		}
+
 		sb.AddIPs(ips)
 	}
 
@@ -330,7 +337,7 @@ func (s *Server) Shutdown(ctx context.Context) error {
 
 	// first, make sure we sync all the changes to the file system holding
 	// the graph root
-	if err := utils.Syncfs(s.Store().GraphRoot()); err != nil {
+	if err := utils.Syncfs(s.ContainerServer.Store().GraphRoot()); err != nil {
 		return fmt.Errorf("failed to sync graph root after shutting down: %w", err)
 	}
 
@@ -344,6 +351,7 @@ func (s *Server) Shutdown(ctx context.Context) error {
 		if err != nil {
 			return fmt.Errorf("failed to write file to indicate a clean shutdown: %w", err)
 		}
+
 		f.Close()
 
 		// finally, attempt to sync the newly created file to disk.
@@ -373,6 +381,7 @@ func getIDMappings(config *libconfig.Config) (*idtools.IDMappings, error) {
 	if err != nil {
 		return nil, err
 	}
+
 	parsedGIDsMappings, err := idtools.ParseIDMap(strings.Split(config.GIDMappings, ","), "GID")
 	if err != nil {
 		return nil, err
@@ -389,9 +398,10 @@ func New(
 	if configIface == nil || configIface.GetData() == nil {
 		return nil, errors.New("provided configuration interface or its data is nil")
 	}
+
 	config := configIface.GetData()
 
-	useDefaultUmask()
+	useDefaultUmask(ctx)
 
 	config.SystemContext.AuthFilePath = config.GlobalAuthFile
 	config.SystemContext.SignaturePolicyPath = config.SignaturePolicyPath
@@ -404,6 +414,7 @@ func New(
 	if err := os.MkdirAll(config.ContainerExitsDir, 0o755); err != nil {
 		return nil, err
 	}
+
 	containerServer, err := lib.New(ctx, configIface)
 	if err != nil {
 		return nil, err
@@ -411,6 +422,7 @@ func New(
 
 	if strings.ToLower(strings.TrimSpace(config.IrqBalanceConfigRestoreFile)) != irqBalanceConfigRestoreDisable {
 		log.Infof(ctx, "Attempting to restore irqbalance config from %s", config.IrqBalanceConfigRestoreFile)
+
 		err = runtimehandlerhooks.RestoreIrqBalanceConfig(context.TODO(), config.IrqBalanceConfigFile, config.IrqBalanceConfigRestoreFile, runtimehandlerhooks.IrqSmpAffinityProcFile)
 		if err != nil {
 			return nil, err
@@ -419,16 +431,20 @@ func New(
 
 	// Check for hostport mapping
 	var hostportManager hostport.HostPortManager
-	if config.RuntimeConfig.DisableHostPortMapping {
+	if config.DisableHostPortMapping {
 		hostportManager = hostport.NewNoopHostportManager()
 	} else {
-		hostportManager = hostport.NewHostportManager()
+		hostportManager, err = hostport.NewMetaHostportManager(ctx)
+		if err != nil {
+			return nil, fmt.Errorf("%w (use --disable-hostport-mapping to disable HostPort handling)", err)
+		}
 	}
 
 	idMappings, err := getIDMappings(config)
 	if err != nil {
 		return nil, err
 	}
+
 	if idMappings != nil {
 		log.Errorf(ctx, "Configuration options 'uid_mappings' and 'gid_mappings' are deprecated, and will be replaced with native Kubernetes support for user namespaces in the future")
 	}
@@ -443,17 +459,21 @@ func New(
 		ContainerServer:          containerServer,
 		hostportManager:          hostportManager,
 		config:                   *config,
+		stream:                   &StreamService{},
 		monitorsChan:             make(chan struct{}),
 		defaultIDMappings:        idMappings,
 		minimumMappableUID:       config.MinimumMappableUID,
 		minimumMappableGID:       config.MinimumMappableGID,
 		pullOperationsInProgress: make(map[pullArguments]*pullOperation),
 		resourceStore:            resourcestore.New(),
+		hooksRetriever:           runtimehandlerhooks.NewHooksRetriever(ctx, config),
 	}
+
 	if s.config.EnablePodEvents {
 		// creating a container events channel only if the evented pleg is enabled
 		s.ContainerEventsChan = make(chan types.ContainerEventResponse, 1000)
 	}
+
 	if err := configureMaxThreads(); err != nil {
 		return nil, err
 	}
@@ -465,6 +485,7 @@ func New(
 	}
 
 	defer devNullFile.Close()
+
 	if err := unix.Dup2(int(devNullFile.Fd()), int(os.Stdin.Fd())); err != nil {
 		return nil, fmt.Errorf("close stdin: %w", err)
 	}
@@ -473,6 +494,7 @@ func New(
 	s.wipeIfAppropriate(ctx, deletedImages)
 
 	var bindAddressStr string
+
 	bindAddress := net.ParseIP(config.StreamAddress)
 	if bindAddress != nil {
 		bindAddressStr = bindAddress.String()
@@ -485,6 +507,7 @@ func New(
 
 	// Prepare streaming server
 	streamServerConfig := streaming.DefaultConfig
+
 	if config.StreamIdleTimeout != "" {
 		idleTimeout, err := time.ParseDuration(config.StreamIdleTimeout)
 		if err != nil {
@@ -493,44 +516,55 @@ func New(
 
 		streamServerConfig.StreamIdleTimeout = idleTimeout
 	}
+
 	streamServerConfig.Addr = net.JoinHostPort(bindAddressStr, config.StreamPort)
+
+	s.stream.streamServerCloseCh = make(chan struct{})
+
 	if config.StreamEnableTLS {
-		certCache := &certConfigCache{
-			tlsCert: config.StreamTLSCert,
-			tlsKey:  config.StreamTLSKey,
-			tlsCA:   config.StreamTLSCA,
-		}
-		// We add the certs to the config, even thought the config is dynamic, because
-		// the http package method, ServeTLS, checks to make sure there is a cert in the
-		// config or it throws an error.
-		cert, err := tls.LoadX509KeyPair(config.StreamTLSCert, config.StreamTLSKey)
+		log.Debugf(ctx, "TLS enabled for streaming server")
+
+		certConf, err := cert.NewCertConfig(ctx, s.stream.streamServerCloseCh, config.StreamTLSCert, config.StreamTLSKey, config.StreamTLSCA)
 		if err != nil {
 			return nil, err
 		}
+		// We add the certs to the config, even thought the config is dynamic, because
+		// the http package method, ServeTLS, checks to make sure there is a certificate in the
+		// config or it throws an error.
+		certificate, err := tls.LoadX509KeyPair(config.StreamTLSCert, config.StreamTLSKey)
+		if err != nil {
+			return nil, fmt.Errorf("load stream server x509 key pair: %w", err)
+		}
+
 		streamServerConfig.TLSConfig = &tls.Config{
-			GetConfigForClient: certCache.GetConfigForClient,
-			Certificates:       []tls.Certificate{cert},
+			GetConfigForClient: certConf.GetConfigForClient,
+			Certificates:       []tls.Certificate{certificate},
 			MinVersion:         tls.VersionTLS12,
 		}
+
+		log.Debugf(ctx, "Applying stream server TLS configuration")
 	}
+
 	s.stream.ctx = ctx
 	s.stream.runtimeServer = s
+
 	s.stream.streamServer, err = streaming.NewServer(streamServerConfig, s.stream)
 	if err != nil {
 		return nil, errors.New("unable to create streaming server")
 	}
 
-	s.stream.streamServerCloseCh = make(chan struct{})
 	go func() {
 		defer close(s.stream.streamServerCloseCh)
+
 		if err := s.stream.streamServer.Start(true); err != nil && !errors.Is(err, http.ErrServerClosed) {
 			log.Fatalf(ctx, "Failed to start streaming server: %v", err)
 		}
 	}()
 
-	log.Debugf(ctx, "Sandboxes: %v", s.ContainerServer.ListSandboxes())
+	log.Debugf(ctx, "Sandboxes: %v", s.ListSandboxes())
 
 	s.startReloadWatcher(ctx)
+
 	if s.config.AutoReloadRegistries {
 		go s.startWatcherForMirrorRegistries(ctx, s.config.SystemContext.SystemRegistriesConfDirPath)
 	}
@@ -540,10 +574,12 @@ func New(
 			return nil, err
 		}
 	} else {
-		logrus.Debug("Metrics are disabled")
+		log.Debugf(ctx, "Metrics are disabled")
 	}
 
-	if err := s.startSeccompNotifierWatcher(ctx); err != nil {
+	if s.config.Seccomp().IsDisabled() {
+		log.Infof(ctx, "Seccomp is disabled. Not starting notifier watcher")
+	} else if err := s.startSeccompNotifierWatcher(ctx); err != nil {
 		return nil, fmt.Errorf("start seccomp notifier watcher: %w", err)
 	}
 
@@ -562,6 +598,10 @@ func New(
 		return nil, err
 	}
 
+	if err := watchdog.New(s.checkCRIHealth).Start(ctx); err != nil {
+		return nil, fmt.Errorf("start systemd watchdog: %w", err)
+	}
+
 	return s, nil
 }
 
@@ -575,20 +615,22 @@ func (s *Server) startReloadWatcher(ctx context.Context) {
 		for {
 			// Block until the signal is received
 			<-ch
+
 			if err := s.config.Reload(ctx); err != nil {
-				logrus.Errorf("Unable to reload configuration: %v", err)
+				log.Errorf(ctx, "Unable to reload configuration: %v", err)
+
 				continue
 			}
 			// ImageServer compiles the list with regex for both
 			// pinned and sandbox/pause images, we need to update them
-			s.StorageImageServer().UpdatePinnedImagesList(append(s.config.PinnedImages, s.config.PauseImage))
-			logrus.Info("Configuration reload completed")
+			s.ContainerServer.StorageImageServer().UpdatePinnedImagesList(append(s.config.PinnedImages, s.config.PauseImage))
+			log.Infof(ctx, "Configuration reload completed")
 			// Print the current configuration.
 			tomlConfig, err := s.config.ToString()
 			if err != nil {
-				logrus.Errorf("Unable to print current configuration: %v", err)
+				log.Errorf(ctx, "Unable to print current configuration: %v", err)
 			} else {
-				logrus.Infof("Current CRI-O configuration:\n%s", tomlConfig)
+				log.Infof(ctx, "Current CRI-O configuration:\n%s", tomlConfig)
 			}
 		}
 	}()
@@ -596,11 +638,12 @@ func (s *Server) startReloadWatcher(ctx context.Context) {
 	log.Infof(ctx, "Registered SIGHUP reload watcher")
 }
 
-func useDefaultUmask() {
+func useDefaultUmask(ctx context.Context) {
 	const defaultUmask = 0o022
+
 	oldUmask := unix.Umask(defaultUmask)
 	if oldUmask != defaultUmask {
-		logrus.Infof(
+		log.Infof(ctx,
 			"Using default umask 0o%#o instead of 0o%#o",
 			defaultUmask, oldUmask,
 		)
@@ -613,9 +656,11 @@ func useDefaultUmask() {
 func (s *Server) wipeIfAppropriate(ctx context.Context, imagesToDelete []storage.StorageImageID) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
+
 	if !s.config.InternalWipe {
 		return
 	}
+
 	var (
 		shouldWipeContainers, shouldWipeImages bool
 		err                                    error
@@ -656,7 +701,8 @@ func (s *Server) wipeIfAppropriate(ctx context.Context, imagesToDelete []storage
 				}
 			}
 		}
-		for _, sb := range s.ContainerServer.ListSandboxes() {
+
+		for _, sb := range s.ListSandboxes() {
 			if err := s.removePodSandbox(ctx, sb); err != nil {
 				log.Warnf(ctx, "Failed to remove sandbox %s: %v", sb.ID(), err)
 			}
@@ -668,7 +714,7 @@ func (s *Server) wipeIfAppropriate(ctx context.Context, imagesToDelete []storage
 	// disk usage gets too high.
 	if shouldWipeImages {
 		for img := range imageMapToDelete {
-			if err := s.StorageImageServer().DeleteImage(s.config.SystemContext, img); err != nil {
+			if err := s.ContainerServer.StorageImageServer().DeleteImage(s.config.SystemContext, img); err != nil {
 				log.Warnf(ctx, "Failed to remove image %s: %v", img, err)
 			}
 		}
@@ -678,59 +724,68 @@ func (s *Server) wipeIfAppropriate(ctx context.Context, imagesToDelete []storage
 func (s *Server) addSandbox(ctx context.Context, sb *sandbox.Sandbox) error {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	return s.ContainerServer.AddSandbox(ctx, sb)
+
+	return s.AddSandbox(ctx, sb)
 }
 
 func (s *Server) getSandbox(ctx context.Context, id string) *sandbox.Sandbox {
 	_, span := log.StartSpan(ctx)
 	defer span.End()
-	return s.ContainerServer.GetSandbox(id)
+
+	return s.GetSandbox(id)
 }
 
 func (s *Server) removeSandbox(ctx context.Context, id string) error {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	return s.ContainerServer.RemoveSandbox(ctx, id)
+
+	return s.RemoveSandbox(ctx, id)
 }
 
 func (s *Server) addContainer(ctx context.Context, c *oci.Container) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	s.ContainerServer.AddContainer(ctx, c)
+
+	s.AddContainer(ctx, c)
 }
 
 func (s *Server) addInfraContainer(ctx context.Context, c *oci.Container) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	s.ContainerServer.AddInfraContainer(ctx, c)
+
+	s.AddInfraContainer(ctx, c)
 }
 
 func (s *Server) getInfraContainer(ctx context.Context, id string) *oci.Container {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	return s.ContainerServer.GetInfraContainer(ctx, id)
+
+	return s.GetInfraContainer(ctx, id)
 }
 
 func (s *Server) removeContainer(ctx context.Context, c *oci.Container) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
+
 	s.ContainerServer.RemoveContainer(ctx, c)
 }
 
 func (s *Server) removeInfraContainer(ctx context.Context, c *oci.Container) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
-	s.ContainerServer.RemoveInfraContainer(ctx, c)
+
+	s.RemoveInfraContainer(ctx, c)
 }
 
 func (s *Server) getPodSandboxFromRequest(ctx context.Context, podSandboxID string) (*sandbox.Sandbox, error) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
+
 	if podSandboxID == "" {
 		return nil, sandbox.ErrIDEmpty
 	}
 
-	sandboxID, err := s.PodIDIndex().Get(podSandboxID)
+	sandboxID, err := s.ContainerServer.PodIDIndex().Get(podSandboxID)
 	if err != nil {
 		return nil, fmt.Errorf("PodSandbox with ID starting with %s not found: %w", podSandboxID, err)
 	}
@@ -739,9 +794,11 @@ func (s *Server) getPodSandboxFromRequest(ctx context.Context, podSandboxID stri
 	if sb == nil {
 		return nil, fmt.Errorf("specified pod sandbox not found: %s", sandboxID)
 	}
+
 	if !sb.Created() {
 		return nil, errSandboxNotCreated
 	}
+
 	return sb, nil
 }
 
@@ -762,7 +819,9 @@ func (s *Server) StartExitMonitor(ctx context.Context) {
 	if err != nil {
 		log.Fatalf(ctx, "Failed to create new watch: %v", err)
 	}
+
 	defer watcher.Close()
+
 	done := make(chan struct{})
 	go s.monitorExits(ctx, watcher, done)
 
@@ -770,6 +829,7 @@ func (s *Server) StartExitMonitor(ctx context.Context) {
 		log.Errorf(ctx, "Watcher.Add(%q) failed: %s", s.config.ContainerExitsDir, err)
 		close(done)
 	}
+
 	<-done
 }
 
@@ -783,6 +843,7 @@ func (s *Server) monitorExits(ctx context.Context, watcher *fsnotify.Watcher, do
 		case <-s.monitorsChan:
 			log.Debugf(ctx, "Closing exit monitor...")
 			close(done)
+
 			return
 		}
 	}
@@ -791,26 +852,38 @@ func (s *Server) monitorExits(ctx context.Context, watcher *fsnotify.Watcher, do
 func (s *Server) handleExit(ctx context.Context, event fsnotify.Event) {
 	ctx, span := log.StartSpan(ctx)
 	defer span.End()
+
 	log.Debugf(ctx, "Event: %v", event)
+
 	if event.Op&fsnotify.Create != fsnotify.Create {
 		return
 	}
+
 	containerID := filepath.Base(event.Name)
 	log.Debugf(ctx, "Container or sandbox exited: %v", containerID)
 	c := s.GetContainer(ctx, containerID)
 	nriCtr := c
 	resource := "container"
+
 	var sb *sandbox.Sandbox
 	if c == nil {
 		sb = s.GetSandbox(containerID)
 		if sb == nil {
 			return
 		}
+
 		c = sb.InfraContainer()
 		resource = "sandbox infra"
+		// We discovered the infra container stopped (potentially unexpectedly).
+		// Since sandboxes status is now being judged by the sb.stopped boolean,
+		// rather than the infra container's status, we have to manually set stopped here.
+		// It's likely we're doing double the work here, but that's better than missing it
+		// if the infra container crashed.
+		sb.SetStopped(ctx, true)
 	} else {
 		sb = s.GetSandbox(c.Sandbox())
 	}
+
 	log.Debugf(ctx, "%s exited and found: %v", resource, containerID)
 
 	if err := s.ContainerStateToDisk(ctx, c); err != nil {
@@ -823,16 +896,14 @@ func (s *Server) handleExit(ctx context.Context, event fsnotify.Event) {
 		}
 	}
 
-	hooks, err := runtimehandlerhooks.GetRuntimeHandlerHooks(ctx, &s.config, sb.RuntimeHandler(), sb.Annotations())
-	if err != nil {
-		log.Warnf(ctx, "Failed to get runtime handler %q hooks", sb.RuntimeHandler())
-	} else if hooks != nil {
+	if hooks := s.hooksRetriever.Get(ctx, sb.RuntimeHandler(), sb.Annotations()); hooks != nil {
 		if err := hooks.PostStop(ctx, c, sb); err != nil {
 			log.Errorf(ctx, "Failed to run post-stop hook for container %s: %v", c.ID(), err)
 		}
 	}
 
 	s.generateCRIEvent(ctx, c, types.ContainerEventType_CONTAINER_STOPPED_EVENT)
+
 	if err := os.Remove(event.Name); err != nil {
 		log.Warnf(ctx, "Failed to remove exit file: %v", err)
 	}
@@ -840,8 +911,8 @@ func (s *Server) handleExit(ctx context.Context, event fsnotify.Event) {
 
 func (s *Server) getSandboxStatuses(ctx context.Context, sandboxID string) (*types.PodSandboxStatus, error) {
 	sandboxStatusRequest := &types.PodSandboxStatusRequest{PodSandboxId: sandboxID}
-	sandboxStatus, err := s.PodSandboxStatus(ctx, sandboxStatusRequest)
 
+	sandboxStatus, err := s.PodSandboxStatus(ctx, sandboxStatusRequest)
 	if isNotFound(err) {
 		return nil, err
 	}
@@ -855,21 +926,26 @@ func (s *Server) getSandboxStatuses(ctx context.Context, sandboxID string) (*typ
 
 func (s *Server) getContainerStatuses(ctx context.Context, sandboxUID string) ([]*types.ContainerStatus, error) {
 	listContainerRequest := &types.ListContainersRequest{Filter: &types.ContainerFilter{LabelSelector: map[string]string{kubetypes.KubernetesPodUIDLabel: sandboxUID}}}
+
 	containers, err := s.ListContainers(ctx, listContainerRequest)
 	if err != nil {
 		return []*types.ContainerStatus{}, err
 	}
 
 	containerStatuses := make([]*types.ContainerStatus, len(containers.GetContainers()))
+
 	for i, cc := range containers.GetContainers() {
-		containerStatusRequest := &types.ContainerStatusRequest{ContainerId: cc.Id}
+		containerStatusRequest := &types.ContainerStatusRequest{ContainerId: cc.GetId()}
+
 		resp, err := s.ContainerStatus(ctx, containerStatusRequest)
 		if isNotFound(err) {
 			continue
 		}
+
 		if err != nil {
 			return []*types.ContainerStatus{}, err
 		}
+
 		containerStatuses[i] = resp.GetStatus()
 	}
 
@@ -878,21 +954,26 @@ func (s *Server) getContainerStatuses(ctx context.Context, sandboxUID string) ([
 
 func (s *Server) getContainerStatusesFromSandboxID(ctx context.Context, sandboxID string) ([]*types.ContainerStatus, error) {
 	listContainerRequest := &types.ListContainersRequest{Filter: &types.ContainerFilter{PodSandboxId: sandboxID}}
+
 	containers, err := s.ListContainers(ctx, listContainerRequest)
 	if err != nil {
 		return []*types.ContainerStatus{}, err
 	}
 
 	containerStatuses := make([]*types.ContainerStatus, len(containers.GetContainers()))
+
 	for i, cc := range containers.GetContainers() {
-		containerStatusRequest := &types.ContainerStatusRequest{ContainerId: cc.Id, Verbose: false}
+		containerStatusRequest := &types.ContainerStatusRequest{ContainerId: cc.GetId(), Verbose: false}
+
 		resp, err := s.ContainerStatus(ctx, containerStatusRequest)
 		if isNotFound(err) {
 			continue
 		}
+
 		if err != nil {
 			return []*types.ContainerStatus{}, err
 		}
+
 		containerStatuses[i] = resp.GetStatus()
 	}
 
@@ -904,8 +985,10 @@ func (s *Server) generateCRIEvent(ctx context.Context, container *oci.Container,
 	if !s.config.EnablePodEvents {
 		return
 	}
-	if err := s.Runtime().UpdateContainerStatus(ctx, container); err != nil {
+
+	if err := s.ContainerServer.Runtime().UpdateContainerStatus(ctx, container); err != nil {
 		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to update the container status %s: %v", eventType, container.ID(), err)
+
 		return
 	}
 
@@ -913,20 +996,21 @@ func (s *Server) generateCRIEvent(ctx context.Context, container *oci.Container,
 		return
 	}
 
-	sandboxStatuses, err := s.getSandboxStatuses(ctx, s.GetSandbox(container.Sandbox()).ID())
-
+	sandboxStatuses, err := s.getSandboxStatuses(ctx, s.ContainerServer.GetSandbox(container.Sandbox()).ID())
 	if isNotFound(err) {
 		return
 	}
 
 	if err != nil {
-		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to get sandbox statuses of the pod %s: %v", eventType, sandboxStatuses.Metadata.Uid, err)
+		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to get sandbox statuses of the pod %s: %v", eventType, sandboxStatuses.GetMetadata().GetUid(), err)
+
 		return
 	}
 
-	containerStatuses, err := s.getContainerStatuses(ctx, sandboxStatuses.Metadata.Uid)
+	containerStatuses, err := s.getContainerStatuses(ctx, sandboxStatuses.GetMetadata().GetUid())
 	if err != nil {
-		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to get container statuses of the pod %s: %v", eventType, sandboxStatuses.Metadata.Uid, err)
+		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to get container statuses of the pod %s: %v", eventType, sandboxStatuses.GetMetadata().GetUid(), err)
+
 		return
 	}
 
@@ -936,6 +1020,7 @@ func (s *Server) generateCRIEvent(ctx context.Context, container *oci.Container,
 	default:
 		log.Errorf(ctx, "GenerateCRIEvent: failed to generate event %s for container %s", eventType, container.ID())
 		metrics.Instance().MetricContainersEventsDroppedInc()
+
 		return
 	}
 }
@@ -945,6 +1030,7 @@ func isNotFound(err error) bool {
 	if !ok {
 		return ok
 	}
+
 	if s.Code() == codes.NotFound {
 		return true
 	}
@@ -958,18 +1044,22 @@ func isNotFound(err error) bool {
 func (s *Server) startWatcherForMirrorRegistries(ctx context.Context, registriesConfDDir string) {
 	if registriesConfDDir == "" {
 		log.Infof(ctx, "No registries.conf.d directory specified, defaulting to /etc/containers/registries.conf.d")
+
 		registriesConfDDir = defaultRegistriesConfDDir
 	}
+
 	watcher, err := fsnotify.NewWatcher()
 	if err != nil {
 		log.Fatalf(ctx, "Failed to create new watcher: %v", err)
 	}
+
 	defer watcher.Close()
 
 	log.Infof(ctx, "Registered reload watcher for mirror registries configuration")
 
 	if err := watcher.Add(registriesConfDDir); err != nil {
 		log.Errorf(ctx, "Failed to add watcher for path %q: %s", registriesConfDDir, err)
+
 		return
 	}
 
@@ -978,7 +1068,9 @@ func (s *Server) startWatcherForMirrorRegistries(ctx context.Context, registries
 
 func (s *Server) watchAndReloadMirrorRegistriesConfiguration(ctx context.Context, watcher *fsnotify.Watcher) {
 	var timer *time.Timer
+
 	reloadChannel := make(chan string, 1)
+
 	go func() {
 		// The for loop ensures that the channel is properly drained, even if
 		// no new events are received, thus preventing potential deadlocks.
@@ -986,6 +1078,7 @@ func (s *Server) watchAndReloadMirrorRegistriesConfiguration(ctx context.Context
 		// an empty string and then reloads the registries.
 		for evenName := range reloadChannel {
 			log.Infof(ctx, "File %q changed, reloading registries configuration", evenName)
+
 			if err := s.config.ReloadRegistries(); err != nil {
 				log.Errorf(ctx, "Failed to reload registry configuration: %v", err)
 			}
@@ -997,11 +1090,14 @@ func (s *Server) watchAndReloadMirrorRegistriesConfiguration(ctx context.Context
 		case event, ok := <-watcher.Events:
 			if !ok {
 				close(reloadChannel)
+
 				return
 			}
+
 			if !strings.HasSuffix(filepath.Base(event.Name), ".conf") {
 				continue
 			}
+
 			if event.Op&(fsnotify.Write|fsnotify.Create|fsnotify.Remove|fsnotify.Chmod) != 0 {
 				// Reset timer if exists, else create a new one.
 				if timer != nil {
@@ -1016,9 +1112,16 @@ func (s *Server) watchAndReloadMirrorRegistriesConfiguration(ctx context.Context
 		case err, ok := <-watcher.Errors:
 			if !ok {
 				close(reloadChannel)
+
 				return
 			}
+
 			log.Errorf(ctx, "Watcher error: %v", err)
 		}
 	}
 }
+
+// ArtifactStore returns a new artifact store instance.
+func (s *Server) ArtifactStore() *ociartifact.Store {
+	return ociartifact.NewStore(s.Store().GraphRoot(), s.Config().SystemContext)
+}
