@@ -4,6 +4,11 @@ set -xe
 TEST_USERNS=${TEST_USERNS:-}
 TEST_KEEP_ON_FAILURE=${TEST_KEEP_ON_FAILURE:-}
 
+if [ -n "$GOCOVERDIR" ]; then
+    # It's used to make coverage profiles. https://go.dev/doc/build-cover
+    mkdir -p "$GOCOVERDIR"
+fi
+
 cd "$(dirname "$(readlink -f "${BASH_SOURCE[0]}")")"
 
 if [[ "$TEST_USERNS" == "1" ]]; then
@@ -39,6 +44,9 @@ fi
 
 # The number of parallel jobs to execute tests
 export JOBS=${JOBS:-$(nproc --all)}
+# The maximum number of additional attempts that will be made on a failed test before it is finally considered failed.
+# https://bats-core.readthedocs.io/en/stable/writing-tests.html#special-variables
+export BATS_TEST_RETRIES=1
 
 bats --version
 
